# sage_setup: distribution = sagemath-categories
r"""
Noetherian rings

EXAMPLES::

    sage: from sage.categories.noetherian_rings import NoetherianRings
    sage: GF(4, "a") in NoetherianRings()                                       # needs sage.rings.finite_rings
    True
    sage: QQ in NoetherianRings()
    True
    sage: ZZ in NoetherianRings()
    True
    sage: IntegerModRing(4) in NoetherianRings()
    True
    sage: IntegerModRing(5) in NoetherianRings()
    True
"""
# ****************************************************************************
#  Copyright (C) 2008 Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2012 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.categories.category import Category
from sage.categories.commutative_rings import CommutativeRings


class NoetherianRings(Category):
    """
    The category of Noetherian rings.

    A Noetherian ring is a commutative ring in which
    every ideal is finitely generated.

    See :wikipedia:`Noetherian ring`

    EXAMPLES::

        sage: from sage.categories.noetherian_rings import NoetherianRings
        sage: C = NoetherianRings(); C
        Category of noetherian rings
        sage: sorted(C.super_categories(), key=str)
        [Category of commutative rings]

    TESTS::

        sage: TestSuite(C).run()
    """
    def super_categories(self):
        """
        EXAMPLES::

            sage: from sage.categories.noetherian_rings import NoetherianRings
            sage: NoetherianRings().super_categories()
            [Category of commutative rings]
        """
        return [CommutativeRings()]

    class ParentMethods:
        def is_noetherian(self, proof=True):
            r"""
            Return ``True``, since this in an object of the category
            of Noetherian rings.

            EXAMPLES::

                sage: ZZ.is_noetherian()
                True
                sage: QQ.is_noetherian()
                True
                sage: ZZ['x'].is_noetherian()
                True
                sage: R.<x> = PolynomialRing(QQ)
                sage: R.is_noetherian()
                True

                sage: L.<z> = LazyLaurentSeriesRing(QQ)                                 # needs sage.combinat
                sage: L.is_noetherian()                                            # needs sage.combinat
                True
            """
            return True

    class ElementMethods:
        pass



# sage_setup: distribution = sagemath-categories
r"""
Vector Spaces
"""
#*****************************************************************************
#  Copyright (C) 2005      David Kohel <kohel@maths.usyd.edu>
#                          William Stein <wstein@math.ucsd.edu>
#                2008      Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2008-2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************
from sage.categories.category import Category
from sage.categories.category_types import Category_module
from sage.categories.category_with_axiom import CategoryWithAxiom_over_base_ring
from sage.categories.cartesian_product import CartesianProductsCategory
from sage.categories.dual import DualObjectsCategory
from sage.categories.tensor import TensorProductsCategory
from sage.categories.filtered_modules import FilteredModulesCategory
from sage.categories.graded_modules import GradedModulesCategory
from sage.categories.fields import Fields
from sage.categories.modules import Modules
from sage.categories.modules_with_basis import ModulesWithBasis
_Fields = Fields()


class VectorSpaces(Category_module):
    """
    The category of (abstract) vector spaces over a given field.

    ??? with an embedding in an ambient vector space ???

    EXAMPLES::

        sage: VectorSpaces(QQ)
        Category of vector spaces over Rational Field
        sage: VectorSpaces(QQ).super_categories()
        [Category of modules over Rational Field]
    """
    @staticmethod
    def __classcall_private__(cls, K, check=True):
        """
        INPUT:

        - ``K`` -- a field
        - ``check`` -- boolean (default: ``True``); whether to check that `K`
          is a field

        EXAMPLES::

            sage: VectorSpaces(QQ) is VectorSpaces(QQ, check=False)
            True

        By default, it is checked that ``K`` is a field::

            sage: VectorSpaces(ZZ)
            Traceback (most recent call last):
            ...
            ValueError: base must be a field or a subcategory of Fields(); got Integer Ring

        With ``check=False``, the check is disabled, possibly enabling
        incorrect inputs::

            sage: VectorSpaces(ZZ, check=False)
            Category of vector spaces over Integer Ring
        """
        if check:
            if not (K in _Fields or
                    (isinstance(K, Category) and K.is_subcategory(_Fields))):
                raise ValueError("base must be a field or a subcategory of Fields();" +
                                 " got {}".format(K))
        return super().__classcall__(cls, K)

    def __init__(self, K):
        """
        EXAMPLES::

            sage: VectorSpaces(QQ)
            Category of vector spaces over Rational Field
            sage: VectorSpaces(ZZ)
            Traceback (most recent call last):
            ...
            ValueError: base must be a field or a subcategory of Fields(); got Integer Ring

        TESTS::

            sage: C = QQ^10      # vector space                                         # needs sage.modules
            sage: TestSuite(C).run()                                                    # needs sage.modules
            sage: TestSuite(VectorSpaces(QQ)).run()
        """
        Category_module.__init__(self, K)

    def _call_(self, x):
        """
        Try to coerce ``x`` into an object of this category.

        EXAMPLES::

            sage: VectorSpaces(QQ)(ZZ^3)                                                # needs sage.modules
            Vector space of dimension 3 over Rational Field

        TESTS:

        Check whether :issue:`30174` is fixed::

            sage: Q3 = FiniteRankFreeModule(QQ, 3)                                      # needs sage.modules
            sage: Modules(QQ)(Q3) is Q3                                                 # needs sage.modules
            True
        """
        try:
            V = x.vector_space(self.base_field())
            if V.base_field() != self.base_field():
                V = V.change_ring(self.base_field())
        except (TypeError, AttributeError) as msg:
            raise TypeError("%s\nunable to coerce x (=%s) into %s" % (msg,x,self))
        return V

    def base_field(self):
        """
        Return the base field over which the vector spaces of this
        category are all defined.

        EXAMPLES::

            sage: VectorSpaces(QQ).base_field()
            Rational Field
        """
        return self.base_ring()

    def super_categories(self):
        """
        EXAMPLES::

            sage: VectorSpaces(QQ).super_categories()
            [Category of modules over Rational Field]
        """
        R = self.base_field()
        return [Modules(R, dispatch=False)]

    def additional_structure(self):
        r"""
        Return  ``None``.

        Indeed, the category of vector spaces defines no additional
        structure: a bimodule morphism between two vector spaces is a
        vector space morphism.

        .. SEEALSO:: :meth:`Category.additional_structure`

        .. TODO:: Should this category be a :class:`CategoryWithAxiom`?

        EXAMPLES::

            sage: VectorSpaces(QQ).additional_structure()
        """
        return None

    class ParentMethods:

        def dimension(self):
            """
            Return the dimension of this vector space.

            EXAMPLES::

                sage: M = FreeModule(FiniteField(19), 100)                              # needs sage.modules
                sage: W = M.submodule([M.gen(50)])                                      # needs sage.modules
                sage: W.dimension()                                                     # needs sage.modules
                1

                sage: M = FiniteRankFreeModule(QQ, 3)                                   # needs sage.modules
                sage: M.dimension()                                                     # needs sage.modules
                3
                sage: M.tensor_module(1, 2).dimension()                                 # needs sage.modules
                27
            """
            return self.rank()

    class ElementMethods:
        pass

    class WithBasis(CategoryWithAxiom_over_base_ring):

        _call_ = ModulesWithBasis.__dict__["_call_"]

        def is_abelian(self):
            """
            Return whether this category is abelian.

            This is always ``True`` since the base ring is a field.

            EXAMPLES::

                sage: VectorSpaces(QQ).WithBasis().is_abelian()
                True
            """
            return True

        class CartesianProducts(CartesianProductsCategory):
            def extra_super_categories(self):
                r"""
                The category of vector spaces with basis is closed under Cartesian products::

                    sage: C = VectorSpaces(QQ).WithBasis()
                    sage: C.CartesianProducts()
                    Category of Cartesian products of vector spaces with basis over Rational Field
                    sage: C in C.CartesianProducts().super_categories()
                    True
                """
                return [self.base_category()]

        class TensorProducts(TensorProductsCategory):
            def extra_super_categories(self):
                r"""
                The category of vector spaces with basis is closed under tensor products::

                    sage: C = VectorSpaces(QQ).WithBasis()
                    sage: C.TensorProducts()
                    Category of tensor products of vector spaces with basis over Rational Field
                    sage: C in C.TensorProducts().super_categories()
                    True
                """
                return [self.base_category()]

        class FiniteDimensional(CategoryWithAxiom_over_base_ring):

            class TensorProducts(TensorProductsCategory):

                def extra_super_categories(self):
                    """
                    Implement the fact that a (finite) tensor product of
                    finite dimensional vector spaces is a finite dimensional vector space.

                    EXAMPLES::

                        sage: VectorSpaces(QQ).WithBasis().FiniteDimensional().TensorProducts().extra_super_categories()
                        [Category of finite dimensional vector spaces with basis over Rational Field]
                        sage: VectorSpaces(QQ).WithBasis().FiniteDimensional().TensorProducts().FiniteDimensional()
                        Category of tensor products of finite dimensional vector spaces with basis over Rational Field
                    """
                    return [self.base_category()]

        class Graded(GradedModulesCategory):
            """
            Category of graded vector spaces with basis.
            """
            def example(self, base_ring=None):
                """
                Return an example of a graded vector space with basis,
                as per :meth:`Category.example()
                <sage.categories.category.Category.example>`.

                EXAMPLES::

                    sage: Modules(QQ).WithBasis().Graded().example()                    # needs sage.combinat sage.modules
                    An example of a graded module with basis:
                     the free module on partitions over Rational Field
                """
                from sage.categories.examples.graded_modules_with_basis import GradedPartitionModule
                if base_ring is None:
                    base_ring = self.base_ring()
                return GradedPartitionModule(base_ring=base_ring)

        class Filtered(FilteredModulesCategory):
            """
            Category of filtered vector spaces with basis.
            """
            def example(self, base_ring=None):
                """
                Return an example of a graded vector space with basis,
                as per :meth:`Category.example()
                <sage.categories.category.Category.example>`.

                EXAMPLES::

                    sage: Modules(QQ).WithBasis().Graded().example()                    # needs sage.combinat sage.modules
                    An example of a graded module with basis:
                     the free module on partitions over Rational Field
                """
                from sage.categories.examples.filtered_modules_with_basis import FilteredPartitionModule
                if base_ring is None:
                    base_ring = self.base_ring()
                return FilteredPartitionModule(base_ring=base_ring)

    class FiniteDimensional(CategoryWithAxiom_over_base_ring):

        class TensorProducts(TensorProductsCategory):

            def extra_super_categories(self):
                """
                Implement the fact that a (finite) tensor product of
                finite dimensional vector spaces is a finite dimensional vector space.

                EXAMPLES::

                    sage: VectorSpaces(QQ).FiniteDimensional().TensorProducts().extra_super_categories()
                    [Category of finite dimensional vector spaces over Rational Field]
                    sage: VectorSpaces(QQ).FiniteDimensional().TensorProducts().FiniteDimensional()
                    Category of tensor products of finite dimensional vector spaces over Rational Field
                """
                return [self.base_category()]

    class DualObjects(DualObjectsCategory):

        def extra_super_categories(self):
            r"""
            Return the dual category.

            EXAMPLES:

            The category of algebras over the Rational Field is dual
            to the category of coalgebras over the same field::

                sage: C = VectorSpaces(QQ)
                sage: C.dual()
                Category of duals of vector spaces over Rational Field
                sage: C.dual().super_categories() # indirect doctest
                [Category of vector spaces over Rational Field]
            """
            return [self.base_category()]

    class CartesianProducts(CartesianProductsCategory):
        def extra_super_categories(self):
            r"""
            The category of vector spaces is closed under Cartesian products::

                sage: C = VectorSpaces(QQ)
                sage: C.CartesianProducts()
                Category of Cartesian products of vector spaces over Rational Field
                sage: C in C.CartesianProducts().super_categories()
                True
            """
            return [self.base_category()]

    class TensorProducts(TensorProductsCategory):
        def extra_super_categories(self):
            r"""
            The category of vector spaces is closed under tensor products::

                sage: C = VectorSpaces(QQ)
                sage: C.TensorProducts()
                Category of tensor products of vector spaces over Rational Field
                sage: C in C.TensorProducts().super_categories()
                True
            """
            return [self.base_category()]

    class Filtered(FilteredModulesCategory):
        """
        Category of filtered vector spaces.
        """

    class Graded(GradedModulesCategory):
        """
        Category of graded vector spaces.
        """



# sage_setup: distribution = sagemath-categories
r"""
Modules
"""
# ****************************************************************************
#  Copyright (C) 2005      David Kohel <kohel@maths.usyd.edu>
#                          William Stein <wstein@math.ucsd.edu>
#                2008      Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2008-2011 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.categories.bimodules import Bimodules
from sage.categories.cartesian_product import CartesianProductsCategory
from sage.categories.category import Category
from sage.categories.category_types import Category_module
from sage.categories.category_with_axiom import CategoryWithAxiom_over_base_ring
from sage.categories.dual import DualObjectsCategory
from sage.categories.fields import Fields
from sage.categories.homset import Hom
from sage.categories.homsets import HomsetsCategory
from sage.categories.morphism import SetMorphism
from sage.categories.sets_cat import Sets
from sage.categories.tensor import TensorProductFunctor, TensorProductsCategory, tensor
from sage.misc.abstract_method import abstract_method
from sage.misc.cachefunc import cached_method
from sage.misc.lazy_import import LazyImport

_Fields = Fields()


class Modules(Category_module):
    r"""
    The category of all modules over a base ring `R`.

    An `R`-module `M` is a left and right `R`-module over a
    commutative ring `R` such that:

    .. MATH::

        r*(x*s) = (r*x)*s \qquad  \forall r,s \in R \text{ and } x \in M

    INPUT:

    - ``base_ring`` -- a ring `R` or subcategory of ``Rings()``
    - ``dispatch`` -- boolean (for internal use; default: ``True``)

    When the base ring is a field, the category of vector spaces is
    returned instead (unless ``dispatch == False``).

    .. WARNING::

        Outside of the context of symmetric modules over a commutative
        ring, the specifications of this category are fuzzy and not
        yet set in stone (see below). The code in this category and
        its subcategories is therefore prone to bugs or arbitrary
        limitations in this case.

    EXAMPLES::

        sage: Modules(ZZ)
        Category of modules over Integer Ring
        sage: Modules(QQ)
        Category of vector spaces over Rational Field

        sage: Modules(Rings())
        Category of modules over rings
        sage: Modules(FiniteFields())
        Category of vector spaces over finite enumerated fields

        sage: Modules(Integers(9))
        Category of modules over Ring of integers modulo 9

        sage: Modules(Integers(9)).super_categories()
        [Category of bimodules over Ring of integers modulo 9 on the left
                                and Ring of integers modulo 9 on the right]

        sage: Modules(ZZ).super_categories()
        [Category of bimodules over Integer Ring on the left
                                and Integer Ring on the right]

        sage: Modules == RingModules
        True

        sage: Modules(ZZ['x']).is_abelian()   # see #6081
        True

    TESTS::

        sage: TestSuite(Modules(ZZ)).run()

    .. TODO::

        - Clarify the distinction, if any, with ``BiModules(R, R)``.
          In particular, if `R` is a commutative ring (e.g. a field),
          some pieces of the code possibly assume that `M` is a
          *symmetric `R`-`R`-bimodule*:

          .. MATH::

              r*x = x*r \qquad  \forall r \in R \text{ and } x \in M

        - Make sure that non symmetric modules are properly supported
          by all the code, and advertise it.

        - Make sure that non commutative rings are properly supported
          by all the code, and advertise it.

        - Add support for base semirings.

        - Implement a ``FreeModules(R)`` category, when so prompted by a
          concrete use case: e.g.  modeling a free module with several
          bases (using :meth:`Sets.SubcategoryMethods.Realizations`)
          or with an atlas of local maps (see e.g. :issue:`15916`).
    """

    @staticmethod
    def __classcall_private__(cls, base_ring, dispatch=True):
        r"""
        Implement the dispatching of ``Modules(field)`` to
        ``VectorSpaces(field)``.

        This feature will later be extended, probably as a covariant
        functorial construction, to support modules over various kinds
        of rings (principal ideal domains, ...), or even over semirings.

        TESTS::

            sage: C = Modules(ZZ); C
            Category of modules over Integer Ring
            sage: C is Modules(ZZ, dispatch = False)
            True
            sage: C is Modules(ZZ, dispatch = True)
            True
            sage: C._reduction
            (<class 'sage.categories.modules.Modules'>, (Integer Ring,), {'dispatch': False})
            sage: TestSuite(C).run()

            sage: Modules(QQ) is VectorSpaces(QQ)
            True
            sage: Modules(QQ, dispatch = True) is VectorSpaces(QQ)
            True

            sage: C = Modules(NonNegativeIntegers()); C   # todo: not implemented
            Category of semiring modules over Non negative integers

            sage: C = Modules(QQ, dispatch = False); C
            Category of modules over Rational Field
            sage: C._reduction
            (<class 'sage.categories.modules.Modules'>, (Rational Field,), {'dispatch': False})
            sage: TestSuite(C).run()
        """
        if dispatch:
            if base_ring in _Fields or (isinstance(base_ring, Category)
                                        and base_ring.is_subcategory(_Fields)):
                from .vector_spaces import VectorSpaces
                return VectorSpaces(base_ring, check=False)
        result = super().__classcall__(cls, base_ring)
        result._reduction[2]['dispatch'] = False
        return result

    def super_categories(self):
        """
        EXAMPLES::

            sage: Modules(ZZ).super_categories()
            [Category of bimodules over Integer Ring on the left
                                    and Integer Ring on the right]

        Nota bene::

            sage: Modules(QQ)
            Category of vector spaces over Rational Field
            sage: Modules(QQ).super_categories()
            [Category of modules over Rational Field]
        """
        R = self.base_ring()
        return [Bimodules(R, R)]

    def additional_structure(self):
        r"""
        Return ``None``.

        Indeed, the category of modules defines no additional structure:
        a bimodule morphism between two modules is a module morphism.

        .. SEEALSO:: :meth:`Category.additional_structure`

        .. TODO:: Should this category be a :class:`~sage.categories.category_with_axiom.CategoryWithAxiom`?

        EXAMPLES::

            sage: Modules(ZZ).additional_structure()
        """
        return None

    class SubcategoryMethods:

        @cached_method
        def base_ring(self):
            r"""
            Return the base ring (category) for ``self``.

            This implements a ``base_ring`` method for all
            subcategories of ``Modules(K)``.

            EXAMPLES::

                sage: C = Modules(QQ) & Semigroups(); C
                Join of Category of semigroups
                    and Category of vector spaces over Rational Field
                sage: C.base_ring()
                Rational Field
                sage: C.base_ring.__module__
                'sage.categories.modules'

                sage: C2 = Modules(Rings()) & Semigroups(); C2
                Join of Category of semigroups and Category of modules over rings
                sage: C2.base_ring()
                Category of rings
                sage: C2.base_ring.__module__
                'sage.categories.modules'

                sage: # needs sage.combinat sage.groups sage.modules
                sage: C3 = DescentAlgebra(QQ,3).B().category()
                sage: C3.base_ring.__module__
                'sage.categories.modules'
                sage: C3.base_ring()
                Rational Field

                sage: # needs sage.combinat sage.modules
                sage: C4 = QuasiSymmetricFunctions(QQ).F().category()
                sage: C4.base_ring.__module__
                'sage.categories.modules'
                sage: C4.base_ring()
                Rational Field
            """
            for C in self.super_categories():
                # Is there a better way to ask if C is a subcategory of Modules?
                if hasattr(C, "base_ring"):
                    return C.base_ring()
            assert False, "some super category of {} should be a category over base ring".format(self)

        def TensorProducts(self):
            r"""
            Return the full subcategory of objects of ``self`` constructed
            as tensor products.

            .. SEEALSO::

                - :class:`.tensor.TensorProductsCategory`
                - :class:`~.covariant_functorial_construction.RegressiveCovariantFunctorialConstruction`.

            EXAMPLES::

                sage: ModulesWithBasis(QQ).TensorProducts()
                Category of tensor products of vector spaces with basis over Rational Field
            """
            return TensorProductsCategory.category_of(self)

        @cached_method
        def DualObjects(self):
            r"""
            Return the category of spaces constructed as duals of
            spaces of ``self``.

            The *dual* of a vector space `V` is the space consisting of
            all linear functionals on `V` (see :wikipedia:`Dual_space`).
            Additional structure on `V` can endow its dual with
            additional structure; for example, if `V` is a finite
            dimensional algebra, then its dual is a coalgebra.

            This returns the category of spaces constructed as dual of
            spaces in ``self``, endowed with the appropriate
            additional structure.

            .. WARNING::

                - This semantic of ``dual`` and ``DualObject`` is
                  imposed on all subcategories, in particular to make
                  ``dual`` a covariant functorial construction.

                  A subcategory that defines a different notion of
                  dual needs to use a different name.

                - Typically, the category of graded modules should
                  define a separate ``graded_dual`` construction (see
                  :issue:`15647`). For now the two constructions are
                  not distinguished which is an oversimplified model.

            .. SEEALSO::

                - :class:`.dual.DualObjectsCategory`
                - :class:`~.covariant_functorial_construction.CovariantFunctorialConstruction`.

            EXAMPLES::

                sage: VectorSpaces(QQ).DualObjects()
                Category of duals of vector spaces over Rational Field

            The dual of a vector space is a vector space::

                sage: VectorSpaces(QQ).DualObjects().super_categories()
                [Category of vector spaces over Rational Field]

            The dual of an algebra is a coalgebra::

                sage: sorted(Algebras(QQ).DualObjects().super_categories(), key=str)
                [Category of coalgebras over Rational Field,
                 Category of duals of vector spaces over Rational Field]

            The dual of a coalgebra is an algebra::

                sage: sorted(Coalgebras(QQ).DualObjects().super_categories(), key=str)
                [Category of algebras over Rational Field,
                 Category of duals of vector spaces over Rational Field]

            As a shorthand, this category can be accessed with the
            :meth:`~Modules.SubcategoryMethods.dual` method::

                sage: VectorSpaces(QQ).dual()
                Category of duals of vector spaces over Rational Field

            TESTS::

                sage: C = VectorSpaces(QQ).DualObjects()
                sage: C.base_category()
                Category of vector spaces over Rational Field
                sage: C.super_categories()
                [Category of vector spaces over Rational Field]
                sage: latex(C)
                \mathbf{DualObjects}(\mathbf{VectorSpaces}_{\Bold{Q}})
                sage: TestSuite(C).run()
            """
            return DualObjectsCategory.category_of(self)

        dual = DualObjects

        @cached_method
        def FiniteDimensional(self):
            r"""
            Return the full subcategory of the finite dimensional objects of ``self``.

            EXAMPLES::

                sage: Modules(ZZ).FiniteDimensional()
                Category of finite dimensional modules over Integer Ring
                sage: Coalgebras(QQ).FiniteDimensional()
                Category of finite dimensional coalgebras over Rational Field
                sage: AlgebrasWithBasis(QQ).FiniteDimensional()
                Category of finite dimensional algebras with basis over Rational Field

            TESTS::

                sage: TestSuite(Modules(ZZ).FiniteDimensional()).run()
                sage: Coalgebras(QQ).FiniteDimensional.__module__
                'sage.categories.modules'
            """
            return self._with_axiom("FiniteDimensional")

        @cached_method
        def FinitelyPresented(self):
            r"""
            Return the full subcategory of the finitely presented objects of ``self``.

            EXAMPLES::

                sage: Modules(ZZ).FinitelyPresented()
                Category of finitely presented modules over Integer Ring
                sage: A = SteenrodAlgebra(2)                                            # needs sage.combinat sage.modules
                sage: from sage.modules.fp_graded.module import FPModule                # needs sage.combinat sage.modules
                sage: FPModule(A, [0, 1], [[Sq(2), Sq(1)]]).category()                  # needs sage.combinat sage.modules
                Category of finitely presented graded modules
                 over mod 2 Steenrod algebra, milnor basis

            TESTS::

                sage: TestSuite(Modules(ZZ).FinitelyPresented()).run()
            """
            return self._with_axiom("FinitelyPresented")

        @cached_method
        def Filtered(self, base_ring=None):
            r"""
            Return the subcategory of the filtered objects of ``self``.

            INPUT:

            - ``base_ring`` -- this is ignored

            EXAMPLES::

                sage: Modules(ZZ).Filtered()
                Category of filtered modules over Integer Ring

                sage: Coalgebras(QQ).Filtered()
                Category of filtered coalgebras over Rational Field

                sage: AlgebrasWithBasis(QQ).Filtered()
                Category of filtered algebras with basis over Rational Field

            .. TODO::

                - Explain why this does not commute with :meth:`WithBasis`
                - Improve the support for covariant functorial
                  constructions categories over a base ring so as to
                  get rid of the ``base_ring`` argument.

            TESTS::

                sage: Coalgebras(QQ).Graded.__module__
                'sage.categories.modules'
            """
            assert base_ring is None or base_ring is self.base_ring()
            from sage.categories.filtered_modules import FilteredModulesCategory
            return FilteredModulesCategory.category_of(self)

        @cached_method
        def Graded(self, base_ring=None):
            r"""
            Return the subcategory of the graded objects of ``self``.

            INPUT:

            - ``base_ring`` -- this is ignored

            EXAMPLES::

                sage: Modules(ZZ).Graded()
                Category of graded modules over Integer Ring

                sage: Coalgebras(QQ).Graded()
                Category of graded coalgebras over Rational Field

                sage: AlgebrasWithBasis(QQ).Graded()
                Category of graded algebras with basis over Rational Field

            .. TODO::

                - Explain why this does not commute with :meth:`WithBasis`
                - Improve the support for covariant functorial
                  constructions categories over a base ring so as to
                  get rid of the ``base_ring`` argument.

            TESTS::

                sage: Coalgebras(QQ).Graded.__module__
                'sage.categories.modules'
            """
            assert base_ring is None or base_ring is self.base_ring()
            from sage.categories.graded_modules import GradedModulesCategory
            return GradedModulesCategory.category_of(self)

        @cached_method
        def Super(self, base_ring=None):
            r"""
            Return the super-analogue category of ``self``.

            INPUT:

            - ``base_ring`` -- this is ignored

            EXAMPLES::

                sage: Modules(ZZ).Super()
                Category of super modules over Integer Ring

                sage: Coalgebras(QQ).Super()
                Category of super coalgebras over Rational Field

                sage: AlgebrasWithBasis(QQ).Super()
                Category of super algebras with basis over Rational Field

            .. TODO::

                - Explain why this does not commute with :meth:`WithBasis`
                - Improve the support for covariant functorial
                  constructions categories over a base ring so as to
                  get rid of the ``base_ring`` argument.

            TESTS::

                sage: Coalgebras(QQ).Super.__module__
                'sage.categories.modules'
            """
            assert base_ring is None or base_ring is self.base_ring()
            from sage.categories.super_modules import SuperModulesCategory
            return SuperModulesCategory.category_of(self)

        @cached_method
        def WithBasis(self):
            r"""
            Return the full subcategory of the objects of ``self`` with
            a distinguished basis.

            EXAMPLES::

                sage: Modules(ZZ).WithBasis()
                Category of modules with basis over Integer Ring
                sage: Coalgebras(QQ).WithBasis()
                Category of coalgebras with basis over Rational Field
                sage: AlgebrasWithBasis(QQ).WithBasis()
                Category of algebras with basis over Rational Field

            TESTS::

                sage: TestSuite(Modules(ZZ).WithBasis()).run()
                sage: Coalgebras(QQ).WithBasis.__module__
                'sage.categories.modules'
            """
            return self._with_axiom("WithBasis")

    class FiniteDimensional(CategoryWithAxiom_over_base_ring):

        def extra_super_categories(self):
            """
            Implement the fact that a finite dimensional module over a finite
            ring is finite.

            EXAMPLES::

                sage: Modules(IntegerModRing(4)).FiniteDimensional().extra_super_categories()
                [Category of finite sets]
                sage: Modules(ZZ).FiniteDimensional().extra_super_categories()
                []
                sage: Modules(GF(5)).FiniteDimensional().is_subcategory(Sets().Finite())
                True
                sage: Modules(ZZ).FiniteDimensional().is_subcategory(Sets().Finite())
                False

                sage: Modules(Rings().Finite()).FiniteDimensional().is_subcategory(Sets().Finite())
                True
                sage: Modules(Rings()).FiniteDimensional().is_subcategory(Sets().Finite())
                False
            """
            base_ring = self.base_ring()
            FiniteSets = Sets().Finite()
            if (isinstance(base_ring, Category) and
                    base_ring.is_subcategory(FiniteSets)) or \
                base_ring in FiniteSets:
                return [FiniteSets]
            else:
                return []

        class TensorProducts(TensorProductsCategory):

            def extra_super_categories(self):
                """
                Implement the fact that a (finite) tensor product of
                finite dimensional modules is a finite dimensional module.

                EXAMPLES::

                    sage: Modules(ZZ).FiniteDimensional().TensorProducts().extra_super_categories()
                    [Category of finite dimensional modules over Integer Ring]
                    sage: Modules(QQ).FiniteDimensional().TensorProducts().FiniteDimensional()
                    Category of tensor products of finite dimensional vector spaces
                     over Rational Field
                """
                return [self.base_category()]

    class FinitelyPresented(CategoryWithAxiom_over_base_ring):

        def extra_super_categories(self):
            """
            Implement the fact that a finitely presented module over a finite
            ring is finite.

            EXAMPLES::

                sage: Modules(IntegerModRing(4)).FiniteDimensional().extra_super_categories()
                [Category of finite sets]
                sage: Modules(ZZ).FiniteDimensional().extra_super_categories()
                []
                sage: Modules(GF(5)).FiniteDimensional().is_subcategory(Sets().Finite())
                True
                sage: Modules(ZZ).FiniteDimensional().is_subcategory(Sets().Finite())
                False

                sage: Modules(Rings().Finite()).FiniteDimensional().is_subcategory(Sets().Finite())
                True
                sage: Modules(Rings()).FiniteDimensional().is_subcategory(Sets().Finite())
                False
            """
            base_ring = self.base_ring()
            FiniteSets = Sets().Finite()
            if (isinstance(base_ring, Category) and
                    base_ring.is_subcategory(FiniteSets)) or \
                base_ring in FiniteSets:
                return [FiniteSets]
            else:
                return []

    Filtered = LazyImport('sage.categories.filtered_modules', 'FilteredModules')
    Graded = LazyImport('sage.categories.graded_modules', 'GradedModules')
    Super = LazyImport('sage.categories.super_modules', 'SuperModules')
    # at_startup currently needed for MatrixSpace, see #22955 (e.g., comment:20)
    WithBasis = LazyImport('sage.categories.modules_with_basis', 'ModulesWithBasis',
                           at_startup=True)

    class ParentMethods:

        def linear_combination(self, iter_of_elements_coeff, factor_on_left=True):
            r"""
            Return the linear combination `\lambda_1 v_1 + \cdots +
            \lambda_k v_k` (resp.  the linear combination `v_1 \lambda_1 +
            \cdots + v_k \lambda_k`) where ``iter_of_elements_coeff`` iterates
            through the sequence `((\lambda_1, v_1), ..., (\lambda_k, v_k))`.

            INPUT:

            - ``iter_of_elements_coeff`` -- iterator of pairs
              ``(element, coeff)`` with ``element`` in ``self`` and
              ``coeff`` in ``self.base_ring()``

            - ``factor_on_left`` -- (optional) if ``True``, the coefficients
              are multiplied from the left; if ``False``, the coefficients
              are multiplied from the right

            EXAMPLES::

                sage: m = matrix([[0,1], [1,1]])                                        # needs sage.modules
                sage: J.<a,b,c> = JordanAlgebra(m)                                      # needs sage.combinat sage.modules
                sage: J.linear_combination(((a+b, 1), (-2*b + c, -1)))                  # needs sage.combinat sage.modules
                1 + (3, -1)
            """
            if factor_on_left:
                return self.sum(coeff * element
                                for element, coeff in iter_of_elements_coeff)
            else:
                return self.sum(element * coeff
                                for element, coeff in iter_of_elements_coeff)

        @cached_method
        def tensor_square(self):
            """
            Return the tensor square of ``self``.

            EXAMPLES::

                sage: A = HopfAlgebrasWithBasis(QQ).example()                           # needs sage.groups sage.modules
                sage: A.tensor_square()                                                 # needs sage.groups sage.modules
                An example of Hopf algebra with basis:
                 the group algebra of the Dihedral group of order 6
                 as a permutation group over Rational Field # An example
                 of Hopf algebra with basis: the group algebra of the Dihedral
                 group of order 6 as a permutation group over Rational Field
            """
            return tensor([self, self])

        def module_morphism(self, *, function, category=None, codomain, **keywords):
            r"""
            Construct a module morphism from ``self`` to ``codomain``.

            Let ``self`` be a module `X` over a ring `R`.
            This constructs a morphism `f: X \to Y`.

            INPUT:

            - ``self`` -- a parent `X` in ``Modules(R)``

            - ``function`` -- a function `f` from `X` to `Y`

            - ``codomain`` -- the codomain `Y` of the morphism (default:
              ``f.codomain()`` if it's defined; otherwise it must be specified)

            - ``category`` -- a category or ``None`` (default: ``None``)

            EXAMPLES::

                sage: # needs sage.modules
                sage: V = FiniteRankFreeModule(QQ, 2)
                sage: e = V.basis('e'); e
                Basis (e_0,e_1) on the
                 2-dimensional vector space over the Rational Field
                sage: neg = V.module_morphism(function=operator.neg, codomain=V); neg
                Generic endomorphism of
                 2-dimensional vector space over the Rational Field
                sage: neg(e[0])
                Element -e_0 of the 2-dimensional vector space over the Rational Field
            """
            # Make sure that we only create a module morphism, even if
            # domain and codomain have more structure
            if category is None:
                category = Modules(self.base_ring())
            return SetMorphism(Hom(self, codomain, category), function)

        def quotient(self, submodule, check=True, **kwds):
            r"""
            Construct the quotient module ``self`` / ``submodule``.

            INPUT:

            - ``submodule`` -- a submodule with basis of ``self``, or
              something that can be turned into one via
              ``self.submodule(submodule)``

            - ``check``, other keyword arguments -- passed on to
              :meth:`quotient_module`.

            This method just delegates to :meth:`quotient_module`.
            Classes implementing modules should override that method.

            Parents in categories with additional structure may override
            :meth:`quotient`. For example, in algebras, :meth:`quotient` will
            be the same as :meth:`quotient_ring`.

            EXAMPLES::

                sage: C = CombinatorialFreeModule(QQ, ['a','b','c'])                    # needs sage.modules
                sage: TA = TensorAlgebra(C)                                             # needs sage.combinat sage.modules
                sage: TA.quotient                                                       # needs sage.combinat sage.modules
                <bound method Rings.ParentMethods.quotient of
                 Tensor Algebra of Free module generated by {'a', 'b', 'c'}
                 over Rational Field>
            """
            return self.quotient_module(submodule, check=check, **kwds)

    class ElementMethods:
        pass

    class Homsets(HomsetsCategory):
        r"""
        The category of homomorphism sets `\hom(X,Y)` for `X`, `Y` modules.
        """

        def extra_super_categories(self):
            """
            EXAMPLES::

                sage: Modules(ZZ).Homsets().extra_super_categories()
                [Category of modules over Integer Ring]
            """
            return [Modules(self.base_category().base_ring())]

        def base_ring(self):
            """
            EXAMPLES::

                sage: Modules(ZZ).Homsets().base_ring()
                Integer Ring

            .. TODO::

                Generalize this so that any homset category of a full
                subcategory of modules over a base ring is a category over
                this base ring.
            """
            return self.base_category().base_ring()

        class ParentMethods:

            @cached_method
            def base_ring(self):
                """
                Return the base ring of ``self``.

                EXAMPLES::

                    sage: # needs sage.modules
                    sage: E = CombinatorialFreeModule(ZZ, [1,2,3])
                    sage: F = CombinatorialFreeModule(ZZ, [2,3,4])
                    sage: H = Hom(E, F)
                    sage: H.base_ring()
                    Integer Ring

                This ``base_ring`` method is actually overridden by
                :meth:`sage.structure.category_object.CategoryObject.base_ring`::

                    sage: H.base_ring.__module__                                        # needs sage.modules

                Here we call it directly::

                    sage: method = H.category().parent_class.base_ring                  # needs sage.modules
                    sage: method.__get__(H)()                                           # needs sage.modules
                    Integer Ring
                """
                return self.domain().base_ring()

            @cached_method
            def zero(self):
                """
                EXAMPLES::

                    sage: # needs sage.modules
                    sage: E = CombinatorialFreeModule(ZZ, [1,2,3])
                    sage: F = CombinatorialFreeModule(ZZ, [2,3,4])
                    sage: H = Hom(E, F)
                    sage: f = H.zero()
                    sage: f
                    Generic morphism:
                      From: Free module generated by {1, 2, 3} over Integer Ring
                      To:   Free module generated by {2, 3, 4} over Integer Ring
                    sage: f(E.monomial(2))
                    0
                    sage: f(E.monomial(3)) == F.zero()
                    True

                TESTS:

                We check that ``H.zero()`` is picklable::

                    sage: loads(dumps(f.parent().zero()))                               # needs sage.modules
                    Generic morphism:
                      From: Free module generated by {1, 2, 3} over Integer Ring
                      To:   Free module generated by {2, 3, 4} over Integer Ring
                """
                from sage.misc.constant_function import ConstantFunction
                return self(ConstantFunction(self.codomain().zero()))

        class Endset(CategoryWithAxiom_over_base_ring):
            """
            The category of endomorphism sets `End(X)` for `X`
            a module (this is not used yet)
            """
            def extra_super_categories(self):
                """
                Implement the fact that the endomorphism set of a module is an algebra.

                .. SEEALSO:: :meth:`CategoryWithAxiom.extra_super_categories`

                EXAMPLES::

                    sage: Modules(ZZ).Endsets().extra_super_categories()
                    [Category of magmatic algebras over Integer Ring]

                    sage: End(ZZ^3) in Algebras(ZZ)                                     # needs sage.modules
                    True
                """
                from .magmatic_algebras import MagmaticAlgebras
                return [MagmaticAlgebras(self.base_category().base_ring())]

    class CartesianProducts(CartesianProductsCategory):
        """
        The category of modules constructed as Cartesian products of modules.

        This construction gives the direct product of modules. The
        implementation is based on the following resources:

        - http://groups.google.fr/group/sage-devel/browse_thread/thread/35a72b1d0a2fc77a/348f42ae77a66d16#348f42ae77a66d16
        - :wikipedia:`Direct_product`
        """
        def extra_super_categories(self):
            """
            A Cartesian product of modules is endowed with a natural
            module structure.

            EXAMPLES::

                sage: Modules(ZZ).CartesianProducts().extra_super_categories()
                [Category of modules over Integer Ring]
                sage: Modules(ZZ).CartesianProducts().super_categories()
                [Category of Cartesian products of commutative additive groups,
                 Category of modules over Integer Ring]
            """
            return [self.base_category()]

        class ParentMethods:

            def __init_extra__(self):
                """
                Initialise the base ring of this Cartesian product.

                EXAMPLES::

                    sage: # needs sage.modules
                    sage: E = CombinatorialFreeModule(ZZ, [1,2,3])
                    sage: F = CombinatorialFreeModule(ZZ, [2,3,4])
                    sage: C = cartesian_product([E, F]); C
                    Free module generated by {1, 2, 3} over Integer Ring (+)
                    Free module generated by {2, 3, 4} over Integer Ring
                    sage: C.base_ring()
                    Integer Ring

                Check that :issue:`29225` is fixed::

                    sage: M = cartesian_product((ZZ^2, ZZ^3)); M                        # needs sage.modules
                    The Cartesian product of
                     (Ambient free module of rank 2 over the principal ideal domain Integer Ring,
                      Ambient free module of rank 3 over the principal ideal domain Integer Ring)
                    sage: M.category()                                                  # needs sage.modules
                    Category of Cartesian products of modules with basis
                     over (Dedekind domains and euclidean domains
                      and noetherian rings
                      and infinite enumerated sets and metric spaces)
                    sage: M.base_ring()                                                 # needs sage.modules
                    Integer Ring

                    sage: A = cartesian_product((QQ^2, QQ['x'])); A                     # needs sage.modules
                    The Cartesian product of
                     (Vector space of dimension 2 over Rational Field,
                      Univariate Polynomial Ring in x over Rational Field)
                    sage: A.category()                                                  # needs sage.modules
                    Category of Cartesian products of vector spaces
                     over (number fields and quotient fields and metric spaces)
                    sage: A.base_ring()                                                 # needs sage.modules
                    Rational Field

                This currently only works if all factors have the same
                base ring::

                    sage: B = cartesian_product((ZZ['x'], QQ^3)); B                     # needs sage.modules
                    The Cartesian product of
                     (Univariate Polynomial Ring in x over Integer Ring,
                      Vector space of dimension 3 over Rational Field)
                    sage: B.category()                                                  # needs sage.modules
                    Category of Cartesian products of commutative additive groups
                    sage: B.base_ring()                                                 # needs sage.modules
                """
                factors = self._sets
                if factors:
                    R = factors[0].base_ring()
                    if all(A.base_ring() is R for A in factors):
                        self._base = R

        class ElementMethods:

            def _lmul_(self, x):
                """
                Return the product of `x` with ``self``.

                EXAMPLES::

                    sage: A = FreeModule(ZZ, 2)                                         # needs sage.modules
                    sage: B = cartesian_product([A, A]); B                              # needs sage.modules
                    The Cartesian product of
                     (Ambient free module of rank 2 over the principal ideal domain Integer Ring,
                      Ambient free module of rank 2 over the principal ideal domain Integer Ring)
                    sage: 5*B(([1, 2], [3, 4]))                                         # needs sage.modules
                    ((5, 10), (15, 20))
                """
                return self.parent()._cartesian_product_of_elements(
                    x * y for y in self.cartesian_factors())

    class TensorProducts(TensorProductsCategory):
        """
        The category of modules constructed by tensor product of modules.
        """
        @cached_method
        def extra_super_categories(self):
            """
            EXAMPLES::

                sage: Modules(ZZ).TensorProducts().extra_super_categories()
                [Category of modules over Integer Ring]
                sage: Modules(ZZ).TensorProducts().super_categories()
                [Category of modules over Integer Ring]
            """
            return [self.base_category()]

        class ParentMethods:
            """
            Implement operations on tensor products of modules.
            """
            def construction(self):
                """
                Return the construction of ``self``.

                EXAMPLES::

                    sage: A = algebras.Free(QQ, 2)                                      # needs sage.combinat sage.modules
                    sage: T = A.tensor(A)                                               # needs sage.combinat sage.modules
                    sage: T.construction()                                              # needs sage.combinat sage.modules
                    (The tensor functorial construction,
                     (Free Algebra on 2 generators (None0, None1) over Rational Field,
                      Free Algebra on 2 generators (None0, None1) over Rational Field))
                """
                try:
                    factors = self.tensor_factors()
                except (TypeError, NotImplementedError):
                    from sage.misc.superseded import deprecation
                    deprecation(34393, "implementations of Modules().TensorProducts() now must define the method tensor_factors")
                    return None
                return (TensorProductFunctor(),
                        factors)

            @abstract_method(optional=True)
            def tensor_factors(self):
                """
                Return the tensor factors of this tensor product.

                EXAMPLES::

                    sage: # needs sage.modules
                    sage: F = CombinatorialFreeModule(ZZ, [1,2])
                    sage: F.rename('F')
                    sage: G = CombinatorialFreeModule(ZZ, [3,4])
                    sage: G.rename('G')
                    sage: T = tensor([F, G]); T
                    F # G
                    sage: T.tensor_factors()
                    (F, G)

                TESTS::

                    sage: Cat = ModulesWithBasis(ZZ).FiniteDimensional().TensorProducts()
                    sage: M = CombinatorialFreeModule(ZZ,                               # needs sage.modules
                    ....:                             ((1, 1), (1, 2), (2, 1), (2, 2)),
                    ....:                             category=Cat)
                    sage: M.construction()                                              # needs sage.modules
                    doctest:warning...
                    DeprecationWarning: implementations of Modules().TensorProducts() now must define the method tensor_factors
                    See https://github.com/sagemath/sage/issues/34393 for details.
                    (VectorFunctor, Integer Ring)
                """



# sage_setup: distribution = sagemath-categories
"""
Coercion methods for categories

The purpose of this Cython module is to hold special coercion methods,
which are inserted by their respective categories.
"""

from sage.structure.element cimport Element
cimport cython


@cython.binding
def _mul_parent(self, other):
    r"""
    Return the product of the two elements, calculated using
    the ``product`` method of the parent.

    This is inserted by :meth:`Magmas.ParentMethods.__init_extra__` as
    default implementation of ``Magmas.ElementMethods._mul_`` if
    ``product`` is implemented in the parent.

    INPUT:

    - ``other`` -- an element of the parent of ``self``

    OUTPUT: an element of the parent of ``self``

    EXAMPLES::

        sage: S = Semigroups().example("free")
        sage: x = S('a'); y = S('b')
        sage: x._mul_parent(y)
        'ab'

    .. SEEALSO::

        - :meth:`Magmas.ElementMethods._mul_parent`
        - :meth:`Magmas.ElementMethods.__init_extra__`
        - :meth:`Magmas.ParentMethods.product`

    This is :meth:`Magmas.ElementMethods._mul_parent`, implemented as
    a Cython method in :mod:`sage.categories.coercion_methods`::

        sage: from sage.cpython.getattr import raw_getattr
        sage: x._mul_parent.__func__ is raw_getattr(Magmas.ElementMethods,
        ....:                                       '_mul_parent')
        True
        sage: x._mul_parent.__func__ is sage.categories.coercion_methods._mul_parent
        True
    """
    return (<Element>self)._parent.product(self, other)



# sage_setup: distribution = sagemath-categories
r"""
Quotient fields
"""
#*****************************************************************************
#  Copyright (C) 2008 Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.category_singleton import Category_singleton
from sage.misc.abstract_method import abstract_method
from sage.categories.fields import Fields

from sage.structure.element import coerce_binop


class QuotientFields(Category_singleton):
    """
    The category of quotient fields over an integral domain.

    EXAMPLES::

        sage: QuotientFields()
        Category of quotient fields
        sage: QuotientFields().super_categories()
        [Category of fields]

    TESTS::

        sage: TestSuite(QuotientFields()).run()
    """

    def super_categories(self):
        """
        EXAMPLES::

            sage: QuotientFields().super_categories()
            [Category of fields]
        """
        return [Fields()]

    class ParentMethods:
        pass

    class ElementMethods:

        @abstract_method
        def numerator(self):
            pass

        @abstract_method
        def denominator(self):
            pass

        @coerce_binop
        def gcd(self, other):
            """
            Greatest common divisor.

            .. NOTE::

                In a field, the greatest common divisor is not very informative,
                as it is only determined up to a unit. But in the fraction field
                of an integral domain that provides both gcd and lcm, it is
                possible to be a bit more specific and define the gcd uniquely
                up to a unit of the base ring (rather than in the fraction
                field).

            AUTHOR:

            - Simon King (2011-02): See :issue:`10771`

            EXAMPLES::

                sage: # needs sage.libs.pari
                sage: R.<x> = QQ['x']
                sage: p = (1+x)^3*(1+2*x^2)/(1-x^5)
                sage: q = (1+x)^2*(1+3*x^2)/(1-x^4)
                sage: factor(p)
                (-2) * (x - 1)^-1 * (x + 1)^3 * (x^2 + 1/2) * (x^4 + x^3 + x^2 + x + 1)^-1
                sage: factor(q)
                (-3) * (x - 1)^-1 * (x + 1) * (x^2 + 1)^-1 * (x^2 + 1/3)
                sage: gcd(p, q)
                (x + 1)/(x^7 + x^5 - x^2 - 1)
                sage: factor(gcd(p, q))
                (x - 1)^-1 * (x + 1) * (x^2 + 1)^-1 * (x^4 + x^3 + x^2 + x + 1)^-1
                sage: factor(gcd(p, 1 + x))
                (x - 1)^-1 * (x + 1) * (x^4 + x^3 + x^2 + x + 1)^-1
                sage: factor(gcd(1 + x, q))
                (x - 1)^-1 * (x + 1) * (x^2 + 1)^-1

            TESTS:

            The following tests that the fraction field returns a correct gcd
            even if the base ring does not provide lcm and gcd::

                sage: # needs sage.libs.pari sage.rings.number_field
                sage: R = ZZ.extension(x^2 + 1, names='i')
                sage: i = R.1
                sage: gcd(5, 3 + 4*i)
                -i - 2
                sage: P.<t> = R[]
                sage: gcd(t, i)
                Traceback (most recent call last):
                ...
                NotImplementedError: Gaussian Integers generated by i
                 in Number Field in i with defining polynomial x^2 + 1
                 does not provide a gcd implementation for univariate polynomials
                sage: q = t/(t + 1); q.parent()
                Fraction Field of Univariate Polynomial Ring in t over Gaussian Integers
                generated by i in Number Field in i with defining polynomial x^2 + 1
                sage: gcd(q, q)
                1
                sage: q.gcd(0)
                1
                sage: (q*0).gcd(0)
                0
            """
            P = self.parent()
            try:
                selfN = self.numerator()
                selfD = self.denominator()
                selfGCD = selfN.gcd(selfD)
                otherN = other.numerator()
                otherD = other.denominator()
                otherGCD = otherN.gcd(otherD)
                selfN = selfN // selfGCD
                selfD = selfD // selfGCD
                otherN = otherN // otherGCD
                otherD = otherD // otherGCD
                tmp = P(selfN.gcd(otherN))/P(selfD.lcm(otherD))
                return tmp
            except (AttributeError, NotImplementedError, TypeError, ValueError):
                zero = P.zero()
                if self == zero and other == zero:
                    return zero
                return P.one()

        @coerce_binop
        def lcm(self, other):
            """
            Least common multiple.

            In a field, the least common multiple is not very informative, as it
            is only determined up to a unit. But in the fraction field of an
            integral domain that provides both gcd and lcm, it is reasonable to
            be a bit more specific and to define the least common multiple so
            that it restricts to the usual least common multiple in the base
            ring and is unique up to a unit of the base ring (rather than up to
            a unit of the fraction field).

            The least common multiple is easily described in terms of the
            prime decomposition. A rational number can be written as a product
            of primes with integer (positive or negative) powers in a unique
            way. The least common multiple of two rational numbers `x` and `y`
            can then be defined by specifying that the exponent of every prime
            `p` in `lcm(x,y)` is the supremum of the exponents of `p` in `x`,
            and the exponent of `p` in `y` (where the primes that does not
            appear in the decomposition of `x` or `y` are considered to have
            exponent zero).


            AUTHOR:

            - Simon King (2011-02): See :issue:`10771`

            EXAMPLES::

                sage: lcm(2/3, 1/5)
                2

            Indeed `2/3 = 2^1 3^{-1} 5^0` and `1/5 = 2^0 3^0
            5^{-1}`, so `lcm(2/3,1/5)= 2^1 3^0 5^0 = 2`.

                sage: lcm(1/3, 1/5)
                1
                sage: lcm(1/3, 1/6)
                1/3

            Some more involved examples::

                sage: # needs sage.libs.pari
                sage: R.<x> = QQ[]
                sage: p = (1+x)^3*(1+2*x^2)/(1-x^5)
                sage: q = (1+x)^2*(1+3*x^2)/(1-x^4)
                sage: factor(p)
                (-2) * (x - 1)^-1 * (x + 1)^3 * (x^2 + 1/2) * (x^4 + x^3 + x^2 + x + 1)^-1
                sage: factor(q)
                (-3) * (x - 1)^-1 * (x + 1) * (x^2 + 1)^-1 * (x^2 + 1/3)
                sage: factor(lcm(p, q))
                (x - 1)^-1 * (x + 1)^3 * (x^2 + 1/3) * (x^2 + 1/2)
                sage: factor(lcm(p, 1 + x))
                (x + 1)^3 * (x^2 + 1/2)
                sage: factor(lcm(1 + x, q))
                (x + 1) * (x^2 + 1/3)

            TESTS:

            The following tests that the fraction field returns a correct lcm
            even if the base ring does not provide lcm and gcd::

                sage: # needs sage.libs.pari sage.rings.number_field
                sage: R = ZZ.extension(x^2+1, names='i')
                sage: i = R.1
                sage: P.<t> = R[]
                sage: lcm(t, i)
                Traceback (most recent call last):
                ...
                NotImplementedError: Gaussian Integers generated by i in Number Field in i
                with defining polynomial x^2 + 1 does not provide a gcd implementation for
                univariate polynomials
                sage: q = t/(t + 1); q.parent()
                Fraction Field of Univariate Polynomial Ring in t over
                 Gaussian Integers generated by i in Number Field in i
                 with defining polynomial x^2 + 1
                sage: lcm(q, q)
                1
                sage: q.lcm(0)
                0
                sage: (q*0).lcm(0)
                0

            Check that it is possible to take lcm of a rational and an integer
            (:issue:`17852`)::

                sage: (1/2).lcm(2)
                2
                sage: type((1/2).lcm(2))
                <class 'sage.rings.rational.Rational'>
            """
            P = self.parent()
            try:
                selfN = self.numerator()
                selfD = self.denominator()
                selfGCD = selfN.gcd(selfD)
                otherN = other.numerator()
                otherD = other.denominator()
                otherGCD = otherN.gcd(otherD)
                selfN = selfN // selfGCD
                selfD = selfD // selfGCD
                otherN = otherN // otherGCD
                otherD = otherD // otherGCD
                return P(selfN.lcm(otherN))/P(selfD.gcd(otherD))
            except (AttributeError, NotImplementedError, TypeError, ValueError):
                zero = P.zero()
                if self == zero or other == zero:
                    return zero
                return P.one()

        @coerce_binop
        def xgcd(self, other):
            """
            Return a triple ``(g,s,t)`` of elements of that field such that
            ``g`` is the greatest common divisor of ``self`` and ``other`` and
            ``g = s*self + t*other``.

            .. NOTE::

                In a field, the greatest common divisor is not very informative,
                as it is only determined up to a unit. But in the fraction field
                of an integral domain that provides both xgcd and lcm, it is
                possible to be a bit more specific and define the gcd uniquely
                up to a unit of the base ring (rather than in the fraction
                field).

            EXAMPLES::

                sage: QQ(3).xgcd(QQ(2))
                (1, 1, -1)
                sage: QQ(3).xgcd(QQ(1/2))
                (1/2, 0, 1)
                sage: QQ(1/3).xgcd(QQ(2))
                (1/3, 1, 0)
                sage: QQ(3/2).xgcd(QQ(5/2))
                (1/2, 2, -1)

                sage: R.<x> = QQ['x']
                sage: p = (1+x)^3*(1+2*x^2)/(1-x^5)
                sage: q = (1+x)^2*(1+3*x^2)/(1-x^4)
                sage: factor(p)                                                         # needs sage.libs.pari
                (-2) * (x - 1)^-1 * (x + 1)^3 * (x^2 + 1/2) * (x^4 + x^3 + x^2 + x + 1)^-1
                sage: factor(q)                                                         # needs sage.libs.pari
                (-3) * (x - 1)^-1 * (x + 1) * (x^2 + 1)^-1 * (x^2 + 1/3)
                sage: g, s, t = xgcd(p, q)
                sage: g
                (x + 1)/(x^7 + x^5 - x^2 - 1)
                sage: g == s*p + t*q
                True

            An example without a well defined gcd or xgcd on its base ring::

                sage: # needs sage.rings.number_field
                sage: K = QuadraticField(5)
                sage: O = K.maximal_order()
                sage: R = PolynomialRing(O, 'x')
                sage: F = R.fraction_field()
                sage: x = F.gen(0)
                sage: x.gcd(x+1)
                1
                sage: x.xgcd(x+1)
                (1, 1/x, 0)
                sage: zero = F.zero()
                sage: zero.gcd(x)
                1
                sage: zero.xgcd(x)
                (1, 0, 1/x)
                sage: zero.xgcd(zero)
                (0, 0, 0)
            """
            P = self.parent()
            try:
                selfN = self.numerator()
                selfD = self.denominator()
                selfGCD = selfN.gcd(selfD)

                otherN = other.numerator()
                otherD = other.denominator()
                otherGCD = otherN.gcd(otherD)

                selfN = selfN // selfGCD
                selfD = selfD // selfGCD
                otherN = otherN // otherGCD
                otherD = otherD // otherGCD

                lcmD = selfD.lcm(otherD)
                g,s,t = selfN.xgcd(otherN)
                return (P(g)/P(lcmD), P(s*selfD)/P(lcmD),P(t*otherD)/P(lcmD))
            except (AttributeError, NotImplementedError, TypeError, ValueError):
                zero = self.parent().zero()
                one = self.parent().one()
                if self != zero:
                    return (one, ~self, zero)
                elif other != zero:
                    return (one, zero, ~other)
                else:
                    return (zero, zero, zero)

        def factor(self, *args, **kwds):
            """
            Return the factorization of ``self`` over the base ring.

            INPUT:

            - ``*args`` -- arbitrary arguments suitable over the base ring
            - ``**kwds`` -- arbitrary keyword arguments suitable over the base ring

            OUTPUT: factorization of ``self`` over the base ring

            EXAMPLES::

                sage: K.<x> = QQ[]
                sage: f = (x^3+x)/(x-3)
                sage: f.factor()                                                        # needs sage.libs.pari
                (x - 3)^-1 * x * (x^2 + 1)

            Here is an example to show that :issue:`7868` has been resolved::

                sage: R.<x,y> = GF(2)[]
                sage: f = x*y/(x+y)
                sage: f.factor()                                                        # needs sage.rings.finite_rings
                (x + y)^-1 * y * x
            """
            return (self.numerator().factor(*args, **kwds) /
                    self.denominator().factor(*args, **kwds))

        def partial_fraction_decomposition(self, decompose_powers=True):
            """
            Decompose fraction field element into a whole part and a list of
            fraction field elements over prime power denominators.

            The sum will be equal to the original fraction.

            INPUT:

            - ``decompose_powers`` -- boolean (default: ``True``);
              whether to decompose prime power denominators as opposed to having
              a single term for each irreducible factor of the denominator

            OUTPUT: partial fraction decomposition of ``self`` over the base ring

            AUTHORS:

            - Robert Bradshaw (2007-05-31)

            EXAMPLES::

                sage: # needs sage.libs.pari
                sage: S.<t> = QQ[]
                sage: q = 1/(t+1) + 2/(t+2) + 3/(t-3); q
                (6*t^2 + 4*t - 6)/(t^3 - 7*t - 6)
                sage: whole, parts = q.partial_fraction_decomposition(); parts
                [3/(t - 3), 1/(t + 1), 2/(t + 2)]
                sage: sum(parts) == q
                True
                sage: q = 1/(t^3+1) + 2/(t^2+2) + 3/(t-3)^5
                sage: whole, parts = q.partial_fraction_decomposition(); parts
                [1/3/(t + 1), 3/(t^5 - 15*t^4 + 90*t^3 - 270*t^2 + 405*t - 243),
                 (-1/3*t + 2/3)/(t^2 - t + 1), 2/(t^2 + 2)]
                sage: sum(parts) == q
                True
                sage: q = 2*t / (t + 3)^2
                sage: q.partial_fraction_decomposition()
                (0, [2/(t + 3), -6/(t^2 + 6*t + 9)])
                sage: for p in q.partial_fraction_decomposition()[1]:
                ....:     print(p.factor())
                (2) * (t + 3)^-1
                (-6) * (t + 3)^-2
                sage: q.partial_fraction_decomposition(decompose_powers=False)
                (0, [2*t/(t^2 + 6*t + 9)])

            We can decompose over a given algebraic extension::

                sage: R.<x> = QQ[sqrt(2)][]                                             # needs sage.rings.number_field sage.symbolic
                sage: r = 1/(x^4+1)                                                     # needs sage.rings.number_field sage.symbolic
                sage: r.partial_fraction_decomposition()                                # needs sage.rings.number_field sage.symbolic
                (0,
                 [(-1/4*sqrt2*x + 1/2)/(x^2 - sqrt2*x + 1),
                  (1/4*sqrt2*x + 1/2)/(x^2 + sqrt2*x + 1)])

                sage: R.<x> = QQ[I][]  # of QQ[sqrt(-1)]                                # needs sage.rings.number_field sage.symbolic
                sage: r =  1/(x^4+1)                                                    # needs sage.rings.number_field sage.symbolic
                sage: r.partial_fraction_decomposition()                                # needs sage.rings.number_field sage.symbolic
                (0, [(-1/2*I)/(x^2 - I), 1/2*I/(x^2 + I)])

            We can also ask Sage to find the least extension where the
            denominator factors in linear terms::

                sage: # needs sage.rings.number_field
                sage: R.<x> = QQ[]
                sage: r = 1/(x^4+2)
                sage: N = r.denominator().splitting_field('a'); N
                Number Field in a with defining polynomial x^8 - 8*x^6 + 28*x^4 + 16*x^2 + 36
                sage: R1.<x1> = N[]
                sage: r1 = 1/(x1^4+2)
                sage: r1.partial_fraction_decomposition()
                (0,
                 [(-1/224*a^6 + 13/448*a^4 - 5/56*a^2 - 25/224)/(x1 - 1/28*a^6 + 13/56*a^4 - 5/7*a^2 - 25/28),
                  (1/224*a^6 - 13/448*a^4 + 5/56*a^2 + 25/224)/(x1 + 1/28*a^6 - 13/56*a^4 + 5/7*a^2 + 25/28),
                  (-5/1344*a^7 + 43/1344*a^5 - 85/672*a^3 - 31/672*a)/(x1 - 5/168*a^7 + 43/168*a^5 - 85/84*a^3 - 31/84*a),
                  (5/1344*a^7 - 43/1344*a^5 + 85/672*a^3 + 31/672*a)/(x1 + 5/168*a^7 - 43/168*a^5 + 85/84*a^3 + 31/84*a)])

            Or we may work directly over an algebraically closed field::

                sage: R.<x> = QQbar[]                                                   # needs sage.rings.number_field
                sage: r =  1/(x^4+1)                                                    # needs sage.rings.number_field
                sage: r.partial_fraction_decomposition()                                # needs sage.rings.number_field
                (0,
                 [(-0.1767766952966369? - 0.1767766952966369?*I)/(x - 0.7071067811865475? - 0.7071067811865475?*I),
                  (-0.1767766952966369? + 0.1767766952966369?*I)/(x - 0.7071067811865475? + 0.7071067811865475?*I),
                  (0.1767766952966369? - 0.1767766952966369?*I)/(x + 0.7071067811865475? - 0.7071067811865475?*I),
                  (0.1767766952966369? + 0.1767766952966369?*I)/(x + 0.7071067811865475? + 0.7071067811865475?*I)])

            We do the best we can over inexact fields::

                sage: # needs sage.rings.number_field sage.rings.real_mpfr
                sage: R.<x> = RealField(20)[]
                sage: q = 1/(x^2 + x + 2)^2 + 1/(x-1); q
                (x^4 + 2.0000*x^3
                  + 5.0000*x^2 + 5.0000*x + 3.0000)/(x^5 + x^4 + 3.0000*x^3 - x^2 - 4.0000)
                sage: whole, parts = q.partial_fraction_decomposition(); parts
                [1.0000/(x - 1.0000),
                 1.0000/(x^4 + 2.0000*x^3 + 5.0000*x^2 + 4.0000*x + 4.0000)]
                sage: sum(parts)
                (x^4 + 2.0000*x^3
                  + 5.0000*x^2 + 5.0000*x + 3.0000)/(x^5 + x^4 + 3.0000*x^3 - x^2 - 4.0000)

            TESTS:

            We test partial fraction for irreducible denominators::

                sage: R.<x> = ZZ[]
                sage: q = x^2/(x-1)
                sage: q.partial_fraction_decomposition()                                # needs sage.libs.pari
                (x + 1, [1/(x - 1)])
                sage: q = x^10/(x-1)^5
                sage: whole, parts = q.partial_fraction_decomposition()                 # needs sage.libs.pari
                sage: whole + sum(parts) == q                                           # needs sage.libs.pari
                True

            And also over finite fields (see :issue:`6052`, :issue:`9945`)::

                sage: R.<x> = GF(2)[]
                sage: q = (x+1)/(x^3+x+1)
                sage: q.partial_fraction_decomposition()                                # needs sage.libs.pari
                (0, [(x + 1)/(x^3 + x + 1)])

                sage: R.<x> = GF(11)[]
                sage: q = x + 1 + 1/(x+1) + x^2/(x^3 + 2*x + 9)
                sage: q.partial_fraction_decomposition()                                # needs sage.libs.pari
                (x + 1, [1/(x + 1), x^2/(x^3 + 2*x + 9)])

            And even the rationals::

                sage: (26/15).partial_fraction_decomposition()
                (1, [1/3, 2/5])
                sage: (26/75).partial_fraction_decomposition()
                (-1, [2/3, 3/5, 2/25])

            A larger example::

                sage: S.<t> = QQ[]
                sage: r = t / (t^3+1)^5
                sage: r.partial_fraction_decomposition()                                # needs sage.libs.pari
                (0,
                 [-35/729/(t + 1),
                  -35/729/(t^2 + 2*t + 1),
                  -25/729/(t^3 + 3*t^2 + 3*t + 1),
                  -4/243/(t^4 + 4*t^3 + 6*t^2 + 4*t + 1),
                  -1/243/(t^5 + 5*t^4 + 10*t^3 + 10*t^2 + 5*t + 1),
                  (35/729*t - 35/729)/(t^2 - t + 1),
                  (25/729*t - 8/729)/(t^4 - 2*t^3 + 3*t^2 - 2*t + 1),
                  (-1/81*t + 5/81)/(t^6 - 3*t^5 + 6*t^4 - 7*t^3 + 6*t^2 - 3*t + 1),
                  (-2/27*t + 1/9)/(t^8 - 4*t^7 + 10*t^6 - 16*t^5 + 19*t^4 - 16*t^3 + 10*t^2 - 4*t + 1),
                  (-2/27*t + 1/27)/(t^10 - 5*t^9 + 15*t^8 - 30*t^7 + 45*t^6 - 51*t^5 + 45*t^4 - 30*t^3 + 15*t^2 - 5*t + 1)])
                sage: sum(r.partial_fraction_decomposition()[1]) == r                   # needs sage.libs.pari
                True

            Some special cases::

                sage: R = Frac(QQ['x']); x = R.gen()
                sage: x.partial_fraction_decomposition()
                (x, [])
                sage: R(0).partial_fraction_decomposition()
                (0, [])
                sage: R(1).partial_fraction_decomposition()
                (1, [])
                sage: (1/x).partial_fraction_decomposition()                            # needs sage.libs.pari
                (0, [1/x])
                sage: (1/x+1/x^3).partial_fraction_decomposition()                      # needs sage.libs.pari
                (0, [1/x, 1/x^3])

            This was fixed in :issue:`16240`::

                sage: # needs sage.libs.pari
                sage: R.<x> = QQ['x']
                sage: p = 1/(-x + 1)
                sage: whole, parts = p.partial_fraction_decomposition()
                sage: p == sum(parts)
                True
                sage: p = 3/(-x^4 + 1)
                sage: whole, parts = p.partial_fraction_decomposition()
                sage: p == sum(parts)
                True
                sage: p = (6*x^2 - 9*x + 5)/(-x^3 + 3*x^2 - 3*x + 1)
                sage: whole, parts = p.partial_fraction_decomposition()
                sage: p == sum(parts)
                True
            """
            denom = self.denominator()
            whole, numer = self.numerator().quo_rem(denom)
            factors = denom.factor()
            if not self.parent().is_exact():
                # factors not grouped in this case
                all = {}
                for r in factors:
                    all[r[0]] = 0
                for r in factors:
                    all[r[0]] += r[1]
                factors = sorted(all.items())

            # TODO(robertwb): Should there be a category of univariate polynomials?
            from sage.rings.fraction_field_element import FractionFieldElement_1poly_field
            is_polynomial_over_field = isinstance(self, FractionFieldElement_1poly_field)

            running_total = 0
            parts = []
            for r, e in factors:
                powers = [1]
                for ee in range(e):
                    powers.append(powers[-1] * r)
                d = powers[e]
                denom_div_d = denom // d
                # We know the inverse exists as the two are relatively prime.
                n = ((numer % d) * denom_div_d.inverse_mod(d)) % d
                if not is_polynomial_over_field:
                    running_total += n * denom_div_d
                # If the multiplicity is not one, further reduce.
                if decompose_powers:
                    r_parts = []
                    for ee in range(e, 0, -1):
                        n, n_part = n.quo_rem(r)
                        if n_part:
                            r_parts.append(n_part/powers[ee])
                    parts.extend(reversed(r_parts))
                else:
                    parts.append(n/powers[e])

            if not is_polynomial_over_field:
                # remainders not unique, need to re-compute whole to take into
                # account this freedom
                whole = (self.numerator() - running_total) // denom
            return whole, parts

        def derivative(self, *args):
            r"""
            The derivative of this rational function, with respect to variables
            supplied in args.

            Multiple variables and iteration counts may be supplied; see
            documentation for the global derivative() function for more
            details.

            .. SEEALSO::

               :meth:`_derivative`

            EXAMPLES::

                sage: F.<x> = Frac(QQ['x'])
                sage: (1/x).derivative()
                -1/x^2

            ::

                sage: (x+1/x).derivative(x, 2)
                2/x^3

            ::

                sage: F.<x,y> = Frac(QQ['x,y'])
                sage: (1/(x+y)).derivative(x,y)
                2/(x^3 + 3*x^2*y + 3*x*y^2 + y^3)
            """
            from sage.misc.derivative import multi_derivative
            return multi_derivative(self, args)

        def _derivative(self, var=None):
            r"""
            Return the derivative of this rational function with respect to the
            variable ``var``.

            Over a ring with a working gcd implementation, the derivative of a
            fraction `f/g`, supposed to be given in lowest terms, is computed as
            `(f'(g/d) - f(g'/d))/(g(g'/d))`, where `d` is a greatest common
            divisor of `f` and `g`.

            INPUT:

            - ``var`` -- variable with respect to which the derivative is computed

            OUTPUT: derivative of ``self`` with respect to ``var``

            .. SEEALSO::

               :meth:`derivative`

            EXAMPLES::

                sage: F.<x> = Frac(QQ['x'])
                sage: t = 1/x^2
                sage: t._derivative(x)
                -2/x^3
                sage: t.derivative()
                -2/x^3

            ::

                sage: F.<x,y> = Frac(QQ['x,y'])
                sage: t = (x*y/(x+y))
                sage: t._derivative(x)
                y^2/(x^2 + 2*x*y + y^2)
                sage: t._derivative(y)
                x^2/(x^2 + 2*x*y + y^2)

            TESTS::

                sage: F.<t> = Frac(ZZ['t'])
                sage: F(0).derivative()
                0
                sage: F(2).derivative()
                0
                sage: t.derivative()
                1
                sage: (1+t^2).derivative()
                2*t
                sage: (1/t).derivative()
                -1/t^2
                sage: ((t+2)/(t-1)).derivative()
                -3/(t^2 - 2*t + 1)
                sage: (t/(1+2*t+t^2)).derivative()
                (-t + 1)/(t^3 + 3*t^2 + 3*t + 1)
            """
            R = self.parent()
            if var in R.gens():
                var = R.ring()(var)

            num = self.numerator()
            den = self.denominator()

            if (num.is_zero()):
                return R.zero()

            if R.is_exact():
                try:
                    numder = num._derivative(var)
                    dender = den._derivative(var)
                    d = den.gcd(dender)
                    den = den // d
                    dender = dender // d
                    tnum = numder * den - num * dender
                    tden = self.denominator() * den
                    if not tden.is_one() and tden.is_unit():
                        try:
                            tnum = tnum * tden.inverse_of_unit()
                            tden = R.ring().one()
                        except AttributeError:
                            pass
                        except NotImplementedError:
                            pass
                    return self.__class__(R, tnum, tden,
                        coerce=False, reduce=False)
                except AttributeError:
                    pass
                except NotImplementedError:
                    pass
                except TypeError:
                    pass
                num = self.numerator()
                den = self.denominator()

            num = num._derivative(var) * den - num * den._derivative(var)
            den = den**2

            return self.__class__(R, num, den,
                coerce=False, reduce=False)



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.graphs sage.combinat
r"""
Supercrystals
"""

#*****************************************************************************
#       Copyright (C) 2017 Franco Saliola <saliola@gmail.com>
#                     2017 Anne Schilling <anne at math.ucdavis.edu>
#                     2019 Travis Scrimshaw <tcscrims at gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.misc.cachefunc import cached_method
from sage.categories.category_singleton import Category_singleton
from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.crystals import Crystals
from sage.categories.tensor import TensorProductsCategory


class SuperCrystals(Category_singleton):
    def super_categories(self):
        r"""
        EXAMPLES::

            sage: from sage.categories.supercrystals import SuperCrystals
            sage: C = SuperCrystals()
            sage: C.super_categories()
            [Category of crystals]
        """
        return [Crystals()]

    class ParentMethods:
        def tensor(self, *crystals, **options):
            """
            Return the tensor product of ``self`` with the crystals ``B``.

            EXAMPLES::

                sage: B = crystals.Letters(['A',[1,2]])
                sage: C = crystals.Tableaux(['A',[1,2]], shape = [2,1])
                sage: T = C.tensor(B); T
                Full tensor product of the crystals
                 [Crystal of BKK tableaux of shape [2, 1] of gl(2|3),
                  The crystal of letters for type ['A', [1, 2]]]
                sage: S = B.tensor(C); S
                Full tensor product of the crystals
                 [The crystal of letters for type ['A', [1, 2]],
                  Crystal of BKK tableaux of shape [2, 1] of gl(2|3)]
                sage: G = T.digraph()
                sage: H = S.digraph()
                sage: G.is_isomorphic(H, edge_labels= True)
                True
            """
            cartan_type = self.cartan_type()
            if any(c.cartan_type() != cartan_type for c in crystals):
                raise ValueError("all crystals must be of the same Cartan type")

            if cartan_type.letter == 'Q':
                from sage.combinat.crystals.tensor_product import FullTensorProductOfQueerSuperCrystals
                return FullTensorProductOfQueerSuperCrystals((self,) + tuple(crystals), **options)
            else:
                from sage.combinat.crystals.tensor_product import FullTensorProductOfSuperCrystals
                return FullTensorProductOfSuperCrystals((self,) + tuple(crystals), **options)

    class Finite(CategoryWithAxiom):
        class ParentMethods:
            @cached_method(key=lambda s,i: tuple(i) if i is not None else s.index_set())
            def digraph(self, index_set=None):
                r"""
                Return the :class:`DiGraph` associated to ``self``.

                EXAMPLES::

                    sage: B = crystals.Letters(['A', [1,3]])
                    sage: G = B.digraph(); G
                    Multi-digraph on 6 vertices
                    sage: Q = crystals.Letters(['Q',3])
                    sage: G = Q.digraph(); G
                    Multi-digraph on 3 vertices
                    sage: G.edges(sort=True)
                    [(1, 2, -1), (1, 2, 1), (2, 3, -2), (2, 3, 2)]

                The edges of the crystal graph are by default colored using
                blue for edge 1, red for edge 2, green for edge 3, and dashed with
                the corresponding color for barred edges. Edge 0 is dotted black::

                    sage: view(G)  # optional - dot2tex graphviz, not tested (opens external window)
                """
                from sage.graphs.digraph import DiGraph
                from sage.misc.latex import LatexExpr
                from sage.combinat.root_system.cartan_type import CartanType

                if index_set is None:
                    index_set = self.index_set()

                G = DiGraph(multiedges=True)
                G.add_vertices(self)
                for i in index_set:
                    for x in G:
                        y = x.f(i)
                        if y is not None:
                            G.add_edge(x, y, i)

                def edge_options(data):
                    u, v, l = data
                    edge_opts = { 'edge_string': '->', 'color': 'black' }
                    if l > 0:
                        edge_opts['color'] = CartanType._colors.get(l, 'black')
                        edge_opts['label'] = LatexExpr(str(l))
                    elif l < 0:
                        edge_opts['color'] = "dashed," + CartanType._colors.get(-l, 'black')
                        edge_opts['label'] = LatexExpr("\\overline{%s}" % str(-l))
                    else:
                        edge_opts['color'] = "dotted," + CartanType._colors.get(l, 'black')
                        edge_opts['label'] = LatexExpr(str(l))
                    return edge_opts

                G.set_latex_options(format='dot2tex', edge_labels=True, edge_options=edge_options)
                return G

            def genuine_highest_weight_vectors(self):
                r"""
                Return the tuple of genuine highest weight elements of ``self``.

                EXAMPLES::

                    sage: B = crystals.Letters(['A', [1,2]])
                    sage: B.genuine_highest_weight_vectors()
                    (-2,)

                    sage: T = B.tensor(B)
                    sage: T.genuine_highest_weight_vectors()
                    ([-2, -1], [-2, -2])
                    sage: s1, s2 = T.connected_components()
                    sage: s = s1 + s2
                    sage: s.genuine_highest_weight_vectors()
                    ([-2, -1], [-2, -2])
                """
                return tuple([x[0] for x in self._genuine_highest_lowest_weight_vectors()])

            connected_components_generators = genuine_highest_weight_vectors

            def connected_components(self):
                r"""
                Return the connected components of ``self`` as subcrystals.

                EXAMPLES::

                    sage: B = crystals.Letters(['A', [1,2]])
                    sage: B.connected_components()
                    [Subcrystal of The crystal of letters for type ['A', [1, 2]]]

                    sage: T = B.tensor(B)
                    sage: T.connected_components()
                    [Subcrystal of Full tensor product of the crystals
                      [The crystal of letters for type ['A', [1, 2]],
                       The crystal of letters for type ['A', [1, 2]]],
                     Subcrystal of Full tensor product of the crystals
                      [The crystal of letters for type ['A', [1, 2]],
                       The crystal of letters for type ['A', [1, 2]]]]
                """
                category = SuperCrystals()
                from sage.categories.regular_supercrystals import RegularSuperCrystals
                if self in RegularSuperCrystals():
                    category = RegularSuperCrystals()
                index_set = self.index_set()
                cartan_type = self.cartan_type()
                CCs = []

                for mg in self.connected_components_generators():
                    if not isinstance(mg, tuple):
                        mg = (mg,)
                    subcrystal = self.subcrystal(generators=mg,
                                                 index_set=index_set,
                                                 cartan_type=cartan_type,
                                                 category=category)
                    CCs.append(subcrystal)

                return CCs

            def genuine_lowest_weight_vectors(self):
                r"""
                Return the tuple of genuine lowest weight elements of ``self``.

                EXAMPLES::

                    sage: B = crystals.Letters(['A', [1,2]])
                    sage: B.genuine_lowest_weight_vectors()
                    (3,)

                    sage: T = B.tensor(B)
                    sage: T.genuine_lowest_weight_vectors()
                    ([3, 3], [3, 2])
                    sage: s1, s2 = T.connected_components()
                    sage: s = s1 + s2
                    sage: s.genuine_lowest_weight_vectors()
                    ([3, 3], [3, 2])
                """
                return tuple([x[1] for x in self._genuine_highest_lowest_weight_vectors()])

            @cached_method
            def _genuine_highest_lowest_weight_vectors(self):
                r"""
                Return the genuine lowest and highest weight elements of ``self``.

                EXAMPLES::

                    sage: B = crystals.Letters(['A', [1,2]])
                    sage: B._genuine_highest_lowest_weight_vectors()
                    ((-2, 3),)

                    sage: T = B.tensor(B)
                    sage: T._genuine_highest_lowest_weight_vectors()
                    (([-2, -1], [3, 3]), ([-2, -2], [3, 2]))
                    sage: s1, s2 = T.connected_components()
                    sage: s = s1 + s2
                    sage: s._genuine_highest_lowest_weight_vectors()
                    (([-2, -1], [3, 3]), ([-2, -2], [3, 2]))

                An example with fake highest/lowest weight elements
                from [BKK2000]_::

                    sage: B = crystals.Tableaux(['A', [1,1]], shape=[3,2,1])
                    sage: B._genuine_highest_lowest_weight_vectors()
                    (([[-2, -2, -2], [-1, -1], [1]], [[-1, 1, 2], [1, 2], [2]]),)
                """
                X = []
                for G in self.digraph().connected_components_subgraphs():
                    src = G.sources()
                    sinks = G.sinks()
                    max_dist = -1
                    pair = None
                    for s in src:
                        for t in sinks:
                            d = G.distance(s, t)
                            if d < float('inf') and d > max_dist:
                                pair = (s, t)
                                max_dist = d
                    X.append(pair)
                return tuple(X)

            def character(self):
                """
                Return the character of ``self``.

                .. TODO::

                    Once the `WeylCharacterRing` is implemented, make this
                    consistent with the implementation in
                    :meth:`sage.categories.classical_crystals.ClassicalCrystals.ParentMethods.character`.

                EXAMPLES::

                    sage: B = crystals.Letters(['A',[1,2]])
                    sage: B.character()
                    B[(1, 0, 0, 0, 0)] + B[(0, 1, 0, 0, 0)] + B[(0, 0, 1, 0, 0)]
                     + B[(0, 0, 0, 1, 0)] + B[(0, 0, 0, 0, 1)]
                """
                from sage.rings.integer_ring import ZZ
                A = self.weight_lattice_realization().algebra(ZZ)
                return A.sum(A(x.weight()) for x in self)

            @cached_method
            def highest_weight_vectors(self):
                """
                Return the highest weight vectors of ``self``.

                EXAMPLES::

                    sage: B = crystals.Letters(['A', [1,2]])
                    sage: B.highest_weight_vectors()
                    (-2,)

                    sage: T = B.tensor(B)
                    sage: T.highest_weight_vectors()
                    ([-2, -2], [-2, -1])

                We give an example from [BKK2000]_ that has fake
                highest weight vectors::

                    sage: B = crystals.Tableaux(['A', [1,1]], shape=[3,2,1])
                    sage: B.highest_weight_vectors()
                    ([[-2, -2, -2], [-1, -1], [1]],
                     [[-2, -2, -2], [-1, 2], [1]],
                     [[-2, -2, 2], [-1, -1], [1]])
                    sage: B.genuine_highest_weight_vectors()
                    ([[-2, -2, -2], [-1, -1], [1]],)
                """
                return tuple(self.digraph().sources())

            @cached_method
            def lowest_weight_vectors(self):
                """
                Return the lowest weight vectors of ``self``.

                EXAMPLES::

                    sage: B = crystals.Letters(['A', [1,2]])
                    sage: B.lowest_weight_vectors()
                    (3,)

                    sage: T = B.tensor(B)
                    sage: sorted(T.lowest_weight_vectors())
                    [[3, 2], [3, 3]]

                We give an example from [BKK2000]_ that has fake
                lowest weight vectors::

                    sage: B = crystals.Tableaux(['A', [1,1]], shape=[3,2,1])
                    sage: sorted(B.lowest_weight_vectors())
                    [[[-2, 1, 2], [-1, 2], [1]],
                     [[-2, 1, 2], [-1, 2], [2]],
                     [[-1, 1, 2], [1, 2], [2]]]
                    sage: B.genuine_lowest_weight_vectors()
                    ([[-1, 1, 2], [1, 2], [2]],)
                """
                return tuple(self.digraph().sinks())

        class ElementMethods:
            def is_genuine_highest_weight(self, index_set=None):
                """
                Return whether ``self`` is a genuine highest weight element.

                INPUT:

                - ``index_set`` -- (optional) the index set of the (sub)crystal
                  on which to check

                EXAMPLES::

                    sage: B = crystals.Tableaux(['A', [1,1]], shape=[3,2,1])
                    sage: for b in B.highest_weight_vectors():
                    ....:     print("{} {}".format(b, b.is_genuine_highest_weight()))
                    [[-2, -2, -2], [-1, -1], [1]] True
                    [[-2, -2, -2], [-1, 2], [1]] False
                    [[-2, -2, 2], [-1, -1], [1]] False
                    sage: [b for b in B if b.is_genuine_highest_weight([-1,0])]
                    [[[-2, -2, -2], [-1, -1], [1]],
                     [[-2, -2, -2], [-1, -1], [2]],
                     [[-2, -2, -2], [-1, 2], [2]],
                     [[-2, -2, 2], [-1, -1], [2]],
                     [[-2, -2, 2], [-1, 2], [2]],
                     [[-2, -2, -2], [-1, 2], [1]],
                     [[-2, -2, 2], [-1, -1], [1]],
                     [[-2, -2, 2], [-1, 2], [1]]]
                """
                P = self.parent()
                if index_set is None or set(index_set) == set(P.index_set()):
                    return self in P.genuine_highest_weight_vectors()
                S = P.subcrystal(generators=P, index_set=index_set, category=P.category())
                return any(self == x.value for x in S.genuine_highest_weight_vectors())

            def is_genuine_lowest_weight(self, index_set=None):
                """
                Return whether ``self`` is a genuine lowest weight element.

                INPUT:

                - ``index_set`` -- (optional) the index set of the (sub)crystal
                  on which to check

                EXAMPLES::

                    sage: B = crystals.Tableaux(['A', [1,1]], shape=[3,2,1])
                    sage: for b in sorted(B.lowest_weight_vectors()):
                    ....:     print("{} {}".format(b, b.is_genuine_lowest_weight()))
                    [[-2, 1, 2], [-1, 2], [1]] False
                    [[-2, 1, 2], [-1, 2], [2]] False
                    [[-1, 1, 2], [1, 2], [2]] True
                    sage: [b for b in B if b.is_genuine_lowest_weight([-1,0])]
                    [[[-2, -1, 1], [-1, 1], [1]],
                     [[-2, -1, 1], [-1, 1], [2]],
                     [[-2, 1, 2], [-1, 1], [2]],
                     [[-2, 1, 2], [-1, 1], [1]],
                     [[-1, -1, 1], [1, 2], [2]],
                     [[-1, -1, 1], [1, 2], [1]],
                     [[-1, 1, 2], [1, 2], [2]],
                     [[-1, 1, 2], [1, 2], [1]]]
                """
                P = self.parent()
                if index_set is None or set(index_set) == set(P.index_set()):
                    return self in P.genuine_lowest_weight_vectors()
                S = P.subcrystal(generators=P, index_set=index_set, category=P.category())
                return any(self == x.value for x in S.genuine_lowest_weight_vectors())

    class TensorProducts(TensorProductsCategory):
        """
        The category of regular crystals constructed by tensor
        product of regular crystals.
        """
        @cached_method
        def extra_super_categories(self):
            """
            EXAMPLES::

                sage: from sage.categories.supercrystals import SuperCrystals
                sage: SuperCrystals().TensorProducts().extra_super_categories()
                [Category of super crystals]
            """
            return [self.base_category()]



# sage_setup: distribution = sagemath-categories
r"""
Schemes
"""

# ****************************************************************************
#       Copyright (C) 2005 David Kohel <kohel@maths.usyd.edu>
#                          William Stein <wstein@math.ucsd.edu>
#                2008-2012 Nicolas M. Thiery <nthiery@users.sf.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.categories.category import Category
from sage.categories.category_types import Category_over_base
from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.sets_cat import Sets
from sage.categories.commutative_additive_groups import CommutativeAdditiveGroups
from sage.categories.rings import Rings
from sage.categories.fields import Fields
from sage.categories.homsets import HomsetsCategory
from sage.misc.abstract_method import abstract_method


class Schemes(Category):
    """
    The category of all schemes.

    EXAMPLES::

        sage: Schemes()
        Category of schemes

    ``Schemes`` can also be used to construct the category of schemes
    over a given base::

        sage: Schemes(Spec(ZZ))
        Category of schemes over Integer Ring

        sage: Schemes(ZZ)
        Category of schemes over Integer Ring

    .. TODO::

        Make ``Schemes()`` a singleton category (and remove
        :class:`Schemes` from the workaround in
        :meth:`.category_types.Category_over_base._test_category_over_bases`).

        This is currently incompatible with the dispatching below.

    TESTS::

        sage: TestSuite(Schemes()).run()

    Check that Hom sets of schemes are in the correct category::

        sage: Schemes().Homsets().super_categories()
        [Category of homsets]
    """
    @staticmethod
    def __classcall_private__(cls, X=None):
        """
        Implement the dispatching ``Schemes(ZZ)`` -> ``Schemes_over_base``.

        EXAMPLES::

            sage: Schemes()
            Category of schemes

            sage: Schemes(Spec(ZZ))
            Category of schemes over Integer Ring

            sage: Schemes(ZZ)
            Category of schemes over Integer Ring
        """
        if X is not None:
            if X not in Schemes():
                X = Schemes()(X)
            return Schemes_over_base(X)
        return super().__classcall__(cls)

    def super_categories(self):
        """
        EXAMPLES::

            sage: Schemes().super_categories()
            [Category of sets]
        """
        return [Sets()]

    def _call_(self, x):
        """
        Construct a scheme from the data in ``x``.

        EXAMPLES:

        Let us first construct the category of schemes::

            sage: S = Schemes(); S
            Category of schemes

        We create a scheme from a ring::

            sage: X = S(ZZ); X                  # indirect doctest
            Spectrum of Integer Ring

        We create a scheme from a scheme (do nothing)::

            sage: S(X)
            Spectrum of Integer Ring

        We create a scheme morphism from a ring homomorphism.x::

            sage: phi = ZZ.hom(QQ); phi
            Natural morphism:
              From: Integer Ring
              To:   Rational Field
            sage: f = S(phi); f                 # indirect doctest
            Affine Scheme morphism:
              From: Spectrum of Rational Field
              To:   Spectrum of Integer Ring
              Defn: Natural morphism:
                      From: Integer Ring
                      To:   Rational Field

            sage: f.domain()
            Spectrum of Rational Field
            sage: f.codomain()
            Spectrum of Integer Ring
            sage: S(f)                          # indirect doctest
            Affine Scheme morphism:
              From: Spectrum of Rational Field
              To:   Spectrum of Integer Ring
              Defn: Natural morphism:
                      From: Integer Ring
                      To:   Rational Field
        """
        from sage.schemes.generic.scheme import Scheme
        if isinstance(x, Scheme):
            return x
        from sage.schemes.generic.morphism import is_SchemeMorphism
        if is_SchemeMorphism(x):
            return x
        from sage.categories.commutative_rings import CommutativeRings
        from sage.schemes.generic.spec import Spec
        from sage.categories.map import Map
        if x in CommutativeRings():
            return Spec(x)
        elif isinstance(x, Map) and x.category_for().is_subcategory(Rings()):
            # x is a morphism of Rings
            A = Spec(x.codomain())
            return A.hom(x)
        else:
            raise TypeError("No way to create an object or morphism in %s from %s" % (self, x))


class Schemes_over_base(Category_over_base):
    """
    The category of schemes over a given base scheme.

    EXAMPLES::

        sage: Schemes(Spec(ZZ))
        Category of schemes over Integer Ring

    TESTS::

        sage: C = Schemes(ZZ)
        sage: TestSuite(C).run()
    """
    def base_scheme(self):
        """
        EXAMPLES::

            sage: Schemes(Spec(ZZ)).base_scheme()
            Spectrum of Integer Ring
        """
        return self.base()

    def super_categories(self):
        """
        EXAMPLES::

            sage: Schemes(Spec(ZZ)).super_categories()
            [Category of schemes]
        """
        return [Schemes()]

    def _repr_object_names(self):
        """
        EXAMPLES::

            sage: Schemes(Spec(ZZ)) # indirect doctest
            Category of schemes over Integer Ring
        """
        from sage.schemes.generic.scheme import AffineScheme
        base = self.base()
        if isinstance(base, AffineScheme):
            base = base.coordinate_ring()
        return f"schemes over {base}"


class AbelianVarieties(Schemes_over_base):
    r"""
    The category of abelian varieties over a given field.

    EXAMPLES::

        sage: AbelianVarieties(QQ)
        Category of abelian varieties over Rational Field
        sage: AbelianVarieties(ZZ)
        Traceback (most recent call last):
        ...
        ValueError: category of abelian varieties is only defined over fields
    """
    def __init__(self, base):
        r"""
        Constructor for the ``AbelianVarieties`` category.

        EXAMPLES::

            sage: AbelianVarieties(QQ)
            Category of abelian varieties over Rational Field
            sage: AbelianVarieties(Spec(QQ))
            Category of abelian varieties over Rational Field
        """
        from sage.schemes.generic.scheme import AffineScheme
        if isinstance(base, AffineScheme):
            base = base.coordinate_ring()
        if base not in Fields():
            raise ValueError('category of abelian varieties is only defined over fields')
        super().__init__(base)

    def base_scheme(self):
        """
        EXAMPLES::

            sage: Schemes(Spec(ZZ)).base_scheme()
            Spectrum of Integer Ring
        """
        base = self.base()
        if base not in Schemes():
            base = Schemes()(base)
        return base

    def super_categories(self):
        """
        EXAMPLES::

            sage: AbelianVarieties(QQ).super_categories()
            [Category of schemes over Rational Field,
             Category of commutative additive groups]
        """
        return [Schemes(self.base_scheme()), CommutativeAdditiveGroups()]

    def _repr_object_names(self):
        """
        EXAMPLES::

            sage: AbelianVarieties(Spec(QQ))  # indirect doctest
            Category of abelian varieties over Rational Field
        """
        return "abelian varieties over %s" % self.base()

    class Homsets(HomsetsCategory):
        r"""
        Overloaded ``Homsets`` class to register the homset
        as an additive abelian group.

        EXAMPLES::

            sage: AbelianVarieties(QQ).Homsets().is_subcategory(CommutativeAdditiveGroups())
            True
        """
        def extra_super_categories(self):
            r"""
            Register the homset as an additive abelian group.

            EXAMPLES::

                sage: Hom(EllipticCurve(j=1), EllipticCurve(j=2)) in CommutativeAdditiveGroups()
                True
            """
            return [CommutativeAdditiveGroups()]

        class Endset(CategoryWithAxiom):
            r"""
            Overloaded ``Endset`` class to register the endset
            as a ring.

            sage: AbelianVarieties(QQ).Endsets().is_subcategory(Rings())
            True
            """
            def extra_super_categories(self):
                r"""
                Register the endset as a ring.

                EXAMPLES::

                    sage: End(EllipticCurve(j=1)) in Rings()
                    True
                """
                return [Rings()]


class Jacobians(Schemes_over_base):
    """
    The category of Jacobians attached to curves or function fields.

    EXAMPLES::

        sage: Jacobians(QQ)
        Category of Jacobians over Rational Field

    TESTS::

        sage: TestSuite(Jacobians(QQ)).run()
    """
    def __init__(self, base):
        r"""
        Constructor of this category.

        EXAMPLES::

            sage: Jacobians(QQ)
            Category of Jacobians over Rational Field
            sage: Jacobians(Spec(QQ))
            Category of Jacobians over Rational Field
        """
        from sage.schemes.generic.scheme import AffineScheme
        if isinstance(base, AffineScheme):
            base = base.coordinate_ring()
        if base not in Fields():
            raise ValueError('category of Jacobians is only defined over fields')
        super().__init__(base)

    def base_scheme(self):
        """
        Return the base scheme of this Jacobians category.

        EXAMPLES::

            sage: Jacobians(QQ).base_scheme()
            Spectrum of Rational Field
        """
        base = self.base()
        if base not in Schemes():
            base = Schemes()(base)
        return base

    def super_categories(self):
        """
        Return the super categories of this Jacobians category.

        EXAMPLES::

            sage: Jacobians(QQ).super_categories()
            [Category of abelian varieties over Rational Field]
        """
        return [AbelianVarieties(self.base_scheme())]

    def _repr_object_names(self):
        """
        Return the string representation of this category.

        EXAMPLES::

            sage: Jacobians(Spec(QQ))  # indirect doctest
            Category of Jacobians over Rational Field
        """
        return "Jacobians over %s" % self.base()

    class ParentMethods:

        @abstract_method
        def base_curve(self):
            """
            Return the curve to which this Jacobian is attached.

            EXAMPLES::

                sage: # needs sage.rings.function_field
                sage: K.<x> = FunctionField(GF(2))
                sage: J = K.jacobian()
                sage: J.base_curve()
                Rational function field in x over Finite Field of size 2
            """



# sage_setup: distribution = sagemath-categories
r"""
Commutative algebra ideals
"""
#*****************************************************************************
#  Copyright (C) 2005      David Kohel <kohel@maths.usyd.edu>
#                          William Stein <wstein@math.ucsd.edu>
#                2008-2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.algebra_ideals import AlgebraIdeals
from sage.categories.category_types import Category_ideal, Category_in_ambient
from sage.categories.commutative_algebras import CommutativeAlgebras
from sage.categories.commutative_rings import CommutativeRings


class CommutativeAlgebraIdeals(Category_ideal):
    """
    The category of ideals in a fixed commutative algebra `A`.

    EXAMPLES::

        sage: C = CommutativeAlgebraIdeals(QQ['x'])
        sage: C
        Category of commutative algebra ideals in
         Univariate Polynomial Ring in x over Rational Field
    """
    def __init__(self, A):
        """
        EXAMPLES::

            sage: CommutativeAlgebraIdeals(ZZ['x'])
            Category of commutative algebra ideals in
             Univariate Polynomial Ring in x over Integer Ring

            sage: CommutativeAlgebraIdeals(ZZ)
            Traceback (most recent call last):
            ...
            TypeError: A (=Integer Ring) must be a commutative algebra

            sage: CommutativeAlgebraIdeals(IntegerModRing(4))
            Traceback (most recent call last):
            ...
            TypeError: A (=Ring of integers modulo 4) must be a commutative algebra

            sage: CommutativeAlgebraIdeals(Partitions(4))                               # needs sage.combinat
            Traceback (most recent call last):
            ...
            TypeError: A (=Partitions of the integer 4) must be a commutative algebra

        TESTS::

            sage: TestSuite(CommutativeAlgebraIdeals(QQ['x'])).run()
        """
        # TODO: replace by ``A in CommutativeAlgebras(*)`` once a
        # suitable mantra has been implemented for this.
        try:
            base_ring = A.base_ring()
        except AttributeError:
            raise TypeError(f"A (={A}) must be a commutative algebra")
        else:
            if base_ring not in CommutativeRings() or A not in CommutativeAlgebras(base_ring.category()):
                raise TypeError(f"A (={A}) must be a commutative algebra")

        Category_in_ambient.__init__(self, A)

    def algebra(self):
        """
        EXAMPLES::

            sage: CommutativeAlgebraIdeals(QQ['x']).algebra()
            Univariate Polynomial Ring in x over Rational Field
        """
        return self.ambient()

    def super_categories(self):
        """
        EXAMPLES::

            sage: CommutativeAlgebraIdeals(QQ['x']).super_categories()
            [Category of algebra ideals in Univariate Polynomial Ring in x over Rational Field]
        """
        R = self.algebra()
        return [AlgebraIdeals(R)]



# sage_setup: distribution = sagemath-categories
r"""
Finite Enumerated Sets
"""
# ****************************************************************************
#  Copyright (C) 2009 Florent Hivert <Florent.Hivert@univ-rouen.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.enumerated_sets import EnumeratedSets
from sage.categories.sets_cat import Sets
from sage.categories.cartesian_product import CartesianProductsCategory
from sage.categories.isomorphic_objects import IsomorphicObjectsCategory
from sage.misc.cachefunc import cached_method
from sage.misc.lazy_import import lazy_import
from sage.cpython.getattr import raw_getattr
lazy_import("sage.rings.integer", "Integer")


class FiniteEnumeratedSets(CategoryWithAxiom):
    """
    The category of finite enumerated sets.

    EXAMPLES::

        sage: FiniteEnumeratedSets()
        Category of finite enumerated sets
        sage: FiniteEnumeratedSets().super_categories()
        [Category of enumerated sets, Category of finite sets]
        sage: FiniteEnumeratedSets().all_super_categories()
        [Category of finite enumerated sets,
         Category of enumerated sets,
         Category of finite sets,
         Category of sets,
         Category of sets with partial maps,
         Category of objects]

    TESTS::

        sage: C = FiniteEnumeratedSets()
        sage: TestSuite(C).run()
        sage: sorted(C.Algebras(QQ).super_categories(), key=str)
        [Category of finite dimensional vector spaces with basis over Rational Field,
         Category of set algebras over Rational Field]

    .. TODO::

        :class:`sage.combinat.debruijn_sequence.DeBruijnSequences` should
        not inherit from this class. If that is solved, then
        :class:`FiniteEnumeratedSets` shall be turned into a subclass of
        :class:`~sage.categories.category_singleton.Category_singleton`.
    """

    def _call_(self, X):
        """
        Construct an object in this category from the data in ``X``.

        EXAMPLES::

            sage: FiniteEnumeratedSets()(GF(3))
            Finite Field of size 3
            sage: Partitions(3)                                                         # needs sage.combinat
            Partitions of the integer 3

        For now, lists, tuples, sets, Sets are coerced into finite
        enumerated sets::

            sage: FiniteEnumeratedSets()([1, 2, 3])
            {1, 2, 3}
            sage: FiniteEnumeratedSets()((1, 2, 3))
            {1, 2, 3}
            sage: FiniteEnumeratedSets()(set([1, 2, 3]))
            {1, 2, 3}
            sage: FiniteEnumeratedSets()(Set([1, 2, 3]))
            {1, 2, 3}
        """
        return EnumeratedSets()._call_(X)

    class ParentMethods:

        def __len__(self):
            """
            Return the number of elements of ``self``.

            EXAMPLES::

                sage: len(GF(5))
                5
                sage: len(MatrixSpace(GF(2), 3, 3))                                     # needs sage.modules
                512
            """
            return int(self.cardinality())

        def _cardinality_from_iterator(self, *ignored_args, **ignored_kwds):
            """
            Return the cardinality of ``self``.

            This brute force implementation of :meth:`cardinality`
            iterates through the elements of ``self`` to count them.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example(); C
                An example of a finite enumerated set: {1,2,3}
                sage: C._cardinality_from_iterator()
                3

            TESTS:

            This is the default implementation of :meth:`cardinality`
            from the category ``FiniteEnumeratedSet()``. To test this,
            we need a fresh example::

                sage: from sage.categories.examples.finite_enumerated_sets import Example
                sage: class FreshExample(Example): pass
                sage: C = FreshExample(); C.rename('FreshExample')
                sage: C.cardinality
                <bound method FiniteEnumeratedSets.ParentMethods._cardinality_from_iterator of FreshExample>

            This method shall return an ``Integer``; we test this
            here, because :meth:`_test_enumerated_set_iter_cardinality`
            does not do it for us::

                sage: type(C._cardinality_from_iterator())
                <class 'sage.rings.integer.Integer'>

            We ignore additional inputs since during doctests classes which
            override ``cardinality()`` call up to the category rather than
            their own ``cardinality()`` method (see :issue:`13688`)::

                sage: C = FiniteEnumeratedSets().example()
                sage: C._cardinality_from_iterator(algorithm='testing')
                3

            Here is a more complete example::

                sage: class TestParent(Parent):
                ....:   def __init__(self):
                ....:       Parent.__init__(self, category=FiniteEnumeratedSets())
                ....:   def __iter__(self):
                ....:       yield 1
                ....:       return
                ....:   def cardinality(self, dummy_arg):
                ....:       return 1 # we don't want to change the semantics of cardinality()
                sage: P = TestParent()
                sage: P.cardinality(-1)
                1
                sage: v = P.list(); v
                [1]
                sage: P.cardinality()
                1
                sage: P.cardinality('use alt algorithm') # Used to break here: see trac #13688
                1
                sage: P.cardinality(dummy_arg='use alg algorithm') # Used to break here: see trac #13688
                1
            """
            c = 0
            for _ in self:
                c += 1
            return Integer(c)

        #Set cardinality to the default implementation
        cardinality = _cardinality_from_iterator

        def _cardinality_from_list(self, *ignored_args, **ignored_kwds):
            """
            The cardinality of ``self``.

            This implementation of :meth:`cardinality` computes the
            cardinality from :meth:`list` (which is
            cached). Reciprocally, calling ``self.list()`` makes this
            method the default implementation of :meth:`cardinality`.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example()
                sage: C._cardinality_from_list()
                3

            We ignore additional inputs since during doctests classes which
            override ``cardinality()`` call up to the category rather than
            their own ``cardinality()`` method (see :issue:`13688`)::

                sage: C = FiniteEnumeratedSets().example()
                sage: C._cardinality_from_list(algorithm='testing')
                3
            """
            return Integer(len(self.tuple()))

        def _unrank_from_list(self, r):
            r"""
            The `r`-th element of ``self``.

            INPUT:

              - ``r`` -- integer between `0` and `n-1`,
                where `n` is the cardinality of ``self``

            OUTPUT: the `r`-th element of ``self``

            This implementation of :meth:`unrank` uses the method
            :meth:`list` (which is cached). Reciprocally, calling
            ``self.list()`` makes this method the default
            implementation of :meth:`unrank`.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example()
                sage: C._unrank_from_list(1)
                2
            """
            lst = self.tuple()
            try:
                return lst[r]
            except IndexError:
                raise ValueError("the value must be in the range from %s to %s" % (0, len(lst) - 1))

        def tuple(self):
            r"""
            Return a :class:`tuple`of the elements of ``self``.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example()
                sage: C.tuple()
                (1, 2, 3)
                sage: C.tuple() is C.tuple()
                True
            """
            # Simpler implementation because it does not have to check whether cardinality is finite
            try: # shortcut
                if self._list is not None:
                    return self._tuple_from_list()
            except AttributeError:
                pass

            if self.list != self._list_default:
                return tuple(self.list())

            return self._tuple_from_iterator()
        _tuple_default = tuple

        def _list_from_iterator(self):
            r"""
            Return a list of the elements of ``self`` after cached.

            It moreover overrides the following methods to use this cache:

            - ``self.__iter__()``
            - ``self.cardinality()``
            - ``self.unrank()``

            .. SEEALSO:: :meth:`_cardinality_from_list`,
                :meth:`_iterator_from_list`, and :meth:`_unrank_from_list`

            .. WARNING::

                The overriding of ``self.__iter__`` to use the cache
                is ignored upon calls such as ``for x in C:`` or
                ``list(C)`` (which essentially ruins its purpose).
                Indeed, Python looks up the ``__iter__`` method
                directly in the class of ``C``, bypassing ``C``'s
                dictionary (see the Python reference manual,
                `Special method lookup for new-style classes <http://docs.python.org/reference/datamodel.html#special-method-lookup-for-new-style-classes>`_)

                Let's take an example::

                    sage: class Example(Parent):
                    ....:     def __init__(self):
                    ....:         Parent.__init__(self, category = FiniteEnumeratedSets())
                    ....:     def __iter__(self):
                    ....:         print("hello!")
                    ....:         for x in [1,2,3]: yield x
                    sage: C = Example()
                    sage: list(C)
                    hello!
                    ...
                    [1, 2, 3]
                    sage: list(C)
                    hello!
                    ...
                    [1, 2, 3]

                Note that ``hello!`` actually gets printed more than once in
                the calls to ``list(C)``. That's because of the
                implicit calls to :meth:`__len__`, which also relies
                on :meth:`__iter__`. Let's call :meth:`list`::

                    sage: C.list()
                    hello!
                    [1, 2, 3]
                    sage: C.list()
                    [1, 2, 3]

                Now we would want the original iterator of ``C`` not
                to be called anymore, but that's not the case::

                    sage: list(C)
                    hello!
                    [1, 2, 3]

            TESTS:

            To test if the caching and overriding works, we need a
            fresh finite enumerated set example, because the caching
            mechanism has already been triggered::

                sage: from sage.categories.examples.finite_enumerated_sets import Example
                sage: class FreshExample(Example): pass
                sage: C = FreshExample(); C.rename('FreshExample')
                sage: C.list
                <bound method EnumeratedSets.ParentMethods.list of FreshExample>
                sage: C.unrank
                <bound method EnumeratedSets.ParentMethods._unrank_from_iterator of FreshExample>
                sage: C.cardinality
                <bound method FiniteEnumeratedSets.ParentMethods._cardinality_from_iterator of FreshExample>
                sage: l1 = C.list(); l1
                [1, 2, 3]
                sage: C.list
                <bound method EnumeratedSets.ParentMethods.list of FreshExample>
                sage: C.unrank
                <bound method FiniteEnumeratedSets.ParentMethods._unrank_from_list of FreshExample>
                sage: C.cardinality
                <bound method FiniteEnumeratedSets.ParentMethods._cardinality_from_list of FreshExample>
                sage: C.__iter__
                <bound method EnumeratedSets.ParentMethods._iterator_from_list of FreshExample>

            We finally check that nothing breaks before and after
            calling explicitly the method ``.list()``::

                sage: class FreshExample(Example): pass
                sage: import __main__; __main__.FreshExample = FreshExample # Fake FreshExample being defined in a python module
                sage: C = FreshExample()
                sage: TestSuite(C).run()
                sage: C.list()
                [1, 2, 3]
                sage: TestSuite(C).run()
            """
            try:
                if self._list is not None:
                    return list(self._list)
            except AttributeError:
                pass
            result = tuple(self.__iter__())
            try:
                self._list = result
                self.__iter__ = self._iterator_from_list
                self.cardinality = self._cardinality_from_list
                self.tuple = self._tuple_from_list
                self.unrank = self._unrank_from_list
            except AttributeError:
                pass
            return list(result)

        def unrank_range(self, start=None, stop=None, step=None):
            r"""
            Return the range of elements of ``self`` starting at ``start``,
            ending at ``stop``, and stepping by ``step``.

            See also ``unrank()``.

            EXAMPLES::

                sage: F = FiniteEnumeratedSet([1,2,3])
                sage: F.unrank_range(1)
                [2, 3]
                sage: F.unrank_range(stop=2)
                [1, 2]
                sage: F.unrank_range(stop=2, step=2)
                [1]
                sage: F.unrank_range(start=1, step=2)
                [2]
                sage: F.unrank_range(stop=-1)
                [1, 2]

                sage: F = FiniteEnumeratedSet([1,2,3,4])
                sage: F.unrank_range(stop=10)
                [1, 2, 3, 4]
            """
            try:
                return list(self._list[start:stop:step])
            except AttributeError:
                pass
            card = self.cardinality() # This may set the list
            try:
                return list(self._list[start:stop:step])
            except AttributeError:
                pass
            if start is None and stop is not None and stop >= 0 and step is None:
                if stop < card:
                    it = self.__iter__()
                    return [next(it) for j in range(stop)]
                return self.list()
            return list(self.tuple()[start:stop:step])

        def iterator_range(self, start=None, stop=None, step=None):
            r"""
            Iterate over the range of elements of ``self`` starting
            at ``start``, ending at ``stop``, and stepping by ``step``.

            .. SEEALSO::

                ``unrank()``, ``unrank_range()``

            EXAMPLES::

                sage: F = FiniteEnumeratedSet([1,2,3])
                sage: list(F.iterator_range(1))
                [2, 3]
                sage: list(F.iterator_range(stop=2))
                [1, 2]
                sage: list(F.iterator_range(stop=2, step=2))
                [1]
                sage: list(F.iterator_range(start=1, step=2))
                [2]
                sage: list(F.iterator_range(start=1, stop=2))
                [2]
                sage: list(F.iterator_range(start=0, stop=1))
                [1]
                sage: list(F.iterator_range(start=0, stop=3, step=2))
                [1, 3]
                sage: list(F.iterator_range(stop=-1))
                [1, 2]

                sage: F = FiniteEnumeratedSet([1,2,3,4])
                sage: list(F.iterator_range(start=1, stop=3))
                [2, 3]
                sage: list(F.iterator_range(stop=10))
                [1, 2, 3, 4]
            """
            L = None
            try:
                L = self._list
            except AttributeError:
                pass
            card = self.cardinality() # This may set the list
            try:
                L = self._list
            except AttributeError:
                pass
            if L is None and start is None and stop is not None and stop >= 0 and step is None:
                if stop < card:
                    it = self.__iter__()
                    for j in range(stop):
                        yield next(it)
                    return
                yield from self
                return
            if L is None:
                L = self.tuple()
            yield from L[start:stop:step]

        def _random_element_from_unrank(self):
            """
            A random element in ``self``.

            ``self.random_element()`` returns a random element in
            ``self`` with uniform probability.

            This is the default implementation from the category
            ``EnumeratedSet()`` which uses the method ``unrank``.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example()
                sage: n = C.random_element()
                sage: n in C
                True

                sage: n = C._random_element_from_unrank()
                sage: n in C
                True

            TODO: implement _test_random which checks uniformness
            """
            from sage.misc.prandom import randint
            c = self.cardinality()
            r = randint(0, c-1)
            return self.unrank(r)
        # Set the default implementation of random_element
        random_element = _random_element_from_unrank

        @cached_method
        def _last_from_iterator(self):
            """
            The last element of ``self``.

            ``self.last()`` returns the last element of ``self``.

            This is the default (brute force) implementation from the
            category ``FiniteEnumeratedSet()`` which can be used when
            the method ``__iter__`` is provided. Its complexity is
            `O(n)` where `n` is the size of ``self``.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example()
                sage: C.last()
                3
                sage: C._last_from_iterator()
                3
            """
            for i in self:
                pass
            return i
        last = _last_from_iterator

        def _last_from_unrank(self):
            """
            The last element of ``self``.

            ``self.last()`` returns the last element of ``self``

            This is a generic implementation from the category
            ``FiniteEnumeratedSet()`` which can be used when the
            method ``unrank`` is provided.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example()
                sage: C._last_from_unrank()
                3
            """
            return self.unrank(self.cardinality() - 1)

        def _test_enumerated_set_iter_cardinality(self, **options):
            """
            Check that the methods :meth:`.cardinality` and
            :meth:`.__iter__` are consistent. Also checks that
            :meth:`.cardinality` returns an ``Integer``.

            For efficiency reasons, those tests are not run if
            :meth:`.cardinality` is
            :meth:`._cardinality_from_iterator`, or if ``self`` is too
            big.

            .. SEEALSO:: :class:`TestSuite`.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().example()
                sage: C._test_enumerated_set_iter_cardinality()

            Let us now break the class::

                sage: from sage.categories.examples.finite_enumerated_sets import Example
                sage: class CCls(Example):
                ....:     def cardinality(self):
                ....:         return 4
                sage: CC = CCls()
                sage: CC._test_enumerated_set_iter_cardinality()
                Traceback (most recent call last):
                ...
                AssertionError: 4 != 3
            """
            tester = self._tester(**options)
            if self.cardinality != self._cardinality_from_iterator:
                card = self.cardinality()
                if card <= tester._max_runs:
                    tester.assertEqual(card,
                                       self._cardinality_from_iterator())

    class CartesianProducts(CartesianProductsCategory):

        def extra_super_categories(self):
            """
            A Cartesian product of finite enumerated sets is a finite
            enumerated set.

            EXAMPLES::

                sage: C = FiniteEnumeratedSets().CartesianProducts()
                sage: C.extra_super_categories()
                [Category of finite enumerated sets]
            """
            return [FiniteEnumeratedSets()]

        class ParentMethods:
            r"""
            TESTS:

            Ideally, these tests should be just after the declaration of the
            associated attributes. But doing this way, Sage will not consider
            them as a doctest.

            We check that Cartesian products of finite enumerated sets
            inherit various methods from `Sets.CartesianProducts`
            and not from :class:`EnumeratedSets.Finite`::

                sage: C = cartesian_product([Partitions(10), Permutations(20)])         # needs sage.combinat
                sage: C in EnumeratedSets().Finite()                                    # needs sage.combinat
                True

                sage: C.random_element.__module__                                       # needs sage.combinat
                'sage.categories.sets_cat'

                sage: C.cardinality.__module__                                          # needs sage.combinat
                'sage.categories.sets_cat'

                sage: C.__iter__.__module__                                             # needs sage.combinat
                'sage.categories.sets_cat'
            """
            random_element = raw_getattr(Sets.CartesianProducts.ParentMethods, "random_element")
            cardinality = raw_getattr(Sets.CartesianProducts.ParentMethods, "cardinality")
            __iter__ = raw_getattr(Sets.CartesianProducts.ParentMethods, "__iter__")

            def last(self):
                r"""
                Return the last element.

                EXAMPLES::

                    sage: C = cartesian_product([Zmod(42), Partitions(10),              # needs sage.combinat
                    ....:                        IntegerRange(5)])
                    sage: C.last()                                                      # needs sage.combinat
                    (41, [1, 1, 1, 1, 1, 1, 1, 1, 1, 1], 4)
                """
                return self._cartesian_product_of_elements(
                        tuple(c.last() for c in self.cartesian_factors()))

            def rank(self, x):
                r"""
                Return the rank of an element of this Cartesian product.

                The *rank* of ``x`` is its position in the
                enumeration. It is an integer between ``0`` and
                ``n-1`` where ``n`` is the cardinality of this set.

                .. SEEALSO::

                    - :meth:`EnumeratedSets.ParentMethods.rank`
                    - :meth:`unrank`

                EXAMPLES::

                    sage: C = cartesian_product([GF(2), GF(11), GF(7)])
                    sage: C.rank(C((1,2,5)))
                    96
                    sage: C.rank(C((0,0,0)))
                    0

                    sage: for c in C: print(C.rank(c))
                    0
                    1
                    2
                    3
                    4
                    5
                    ...
                    150
                    151
                    152
                    153

                    sage: # needs sage.combinat
                    sage: F1 = FiniteEnumeratedSet('abcdefgh')
                    sage: F2 = IntegerRange(250)
                    sage: F3 = Partitions(20)
                    sage: C = cartesian_product([F1, F2, F3])
                    sage: c = C(('a', 86, [7,5,4,4]))
                    sage: C.rank(c)
                    54213
                    sage: C.unrank(54213)
                    ('a', 86, [7, 5, 4, 4])
                """
                from sage.rings.integer_ring import ZZ
                x = self(x)
                b = ZZ.one()
                rank = ZZ.zero()
                for f, c in zip(reversed(x.cartesian_factors()),
                                reversed(self.cartesian_factors())):
                    rank += b * c.rank(f)
                    b *= c.cardinality()
                return rank

            def unrank(self, i):
                r"""
                Return the `i`-th element of this Cartesian product.

                INPUT:

                - ``i`` -- integer between `0` and `n-1` where
                  `n` is the cardinality of this set

                .. SEEALSO::

                    - :meth:`EnumeratedSets.ParentMethods.unrank`
                    - :meth:`rank`

                EXAMPLES::

                    sage: C = cartesian_product([GF(3), GF(11), GF(7), GF(5)])
                    sage: c = C.unrank(123); c
                    (0, 3, 3, 3)
                    sage: C.rank(c)
                    123

                    sage: c = C.unrank(857); c
                    (2, 2, 3, 2)
                    sage: C.rank(c)
                    857

                    sage: C.unrank(2500)
                    Traceback (most recent call last):
                    ...
                    IndexError: index i (=2) is greater than the cardinality
                """
                from sage.rings.integer_ring import ZZ
                i = ZZ(i)
                if i < 0:
                    raise IndexError("i (={}) must be a nonnegative integer")
                elt = []
                for c in reversed(self.cartesian_factors()):
                    card = c.cardinality()
                    elt.insert(0, c.unrank(i % card))
                    i //= card
                if i:
                    raise IndexError("index i (={}) is greater than the cardinality".format(i))
                return self._cartesian_product_of_elements(elt)

    class IsomorphicObjects(IsomorphicObjectsCategory):

        def example(self):
            """
            Return an example of isomorphic object of a finite
            enumerated set, as per :meth:`Category.example
            <sage.categories.category.Category.example>`.

            EXAMPLES::

                sage: FiniteEnumeratedSets().IsomorphicObjects().example()
                The image by some isomorphism of An example of a finite enumerated set: {1,2,3}
            """
            from sage.categories.examples.finite_enumerated_sets import IsomorphicObjectOfFiniteEnumeratedSet
            return IsomorphicObjectOfFiniteEnumeratedSet()

        class ParentMethods:

            def cardinality(self):
                r"""
                Return the cardinality of ``self`` which is the same
                as that of the ambient set ``self`` is isomorphic to.

                EXAMPLES::

                    sage: A = FiniteEnumeratedSets().IsomorphicObjects().example(); A
                    The image by some isomorphism of An example of a finite enumerated set: {1,2,3}
                    sage: A.cardinality()
                    3
                """
                return self.ambient().cardinality()

            def __iter__(self):
                r"""
                Return an iterator over ``self``, using the bijection
                with the ambient space.

                EXAMPLES::

                    sage: A = FiniteEnumeratedSets().IsomorphicObjects().example(); A
                    The image by some isomorphism of An example of a finite enumerated set: {1,2,3}
                    sage: list(A)                  # indirect doctest
                    [1, 4, 9]
                """
                for x in self.ambient():
                    yield self.retract(x)
